/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#pragma once

#include "stdafx.h"
#include <windows.h>
#include <crtdbg.h>
#include "IntrFace.h"
#include "NewCaps.h"

#include <vector>
#include <string>
using namespace std;

#include "ResourceList.h"

//////////////////////////////////////////////////////////////////////////////
//
//	class CPciFunction
//
//////////////////////////////////////////////////////////////////////////////

class CPciFunction
{
public:
	/////////////////////////////
	// Constructor/Destructor.
	/////////////////////////////

	CPciFunction(HANDLE hDriver);	// This constructor creates a Host/PCI bridge.
	CPciFunction(HANDLE hDriver, ULONG ulBus, ULONG ulDevice, ULONG ulFunction);
	~CPciFunction();

	/////////////////////////////
	// Public interface.
	/////////////////////////////

	CPciFunction * Find(ULONG ulBus, ULONG ulDevice, ULONG ulFunction);
	BOOL IsBridge();
	PTYPEX_HEADER GetPciHeader(){ return &m_TypeXHeader; }
	void WriteChanges();
	void Dump();

	string GetVendorIdString(){ return m_strVendorId; }
	string GetDeviceIdString(){ return m_strDeviceId; }
	string GetSubSystemVendorIdString(){ return m_strSubSystemVendorId; }

	vector<CPciFunction*> * GetSecondaryBusPciFunctions(){ return &m_vectSecondaryBusPciFunctions; }

	PCAPABILITIES_HEADER GetFirstCapabilitiesHeader(DWORD *pdwDwordOffset);
	PCAPABILITIES_HEADER GetNextCapabilitiesHeader(PCAPABILITIES_HEADER pCapsHdr, DWORD *pdwDwordOffset);

	BOOL ReadDeviceData(DWORD dwBarIndex, DWORD dwOffset, PVOID pBuffer, DWORD dwItemSize, DWORD dwItemCount);
	BOOL WriteDeviceData(DWORD dwBarIndex, DWORD dwOffset, PVOID pBuffer, DWORD dwItemSize, DWORD dwItemCount);

	/////////////////////////////
	// Public member variables.
	/////////////////////////////

	// This is our bus/device/function.
	ULONG m_ulBus;
	ULONG m_ulDevice;
	ULONG m_ulFunction;

	// This is the BAR size as returned either 
	// from the registry or from a BAR probe.
	DWORD m_vdwBarSizes[6];

private:
	/////////////////////////////
	// Member variables.
	/////////////////////////////

	// This is the handle to our kernel-mode driver.
	HANDLE m_hDriver;

	// Our function could be of either type0, type1 or type2.
	TYPEX_HEADER m_TypeXHeader;

	// This is an copy of the configuration space as it was read in from the PCI bus.
	// We use it to know which fields to write to the device once the "WriteChanges"
	// member function is being called. 
	TYPEX_HEADER m_OriginalTypeXHeader;

	// If we are a bridge we enumerate all functions on our secondary bus by calling our 
	// m_EnumerateSecondaryBusFunctions function. The m_EnumerateSecondaryBusFunctions 
	// function stores all CPciFunction objects found in this vector.
	vector<CPciFunction*> m_vectSecondaryBusPciFunctions;

	// This is our vendor/device ID strings as read in from the pcidevs.txt file.
	// The strings are empty if the Vendor/device IDs were not found in the file.
	string m_strVendorId;
	string m_strDeviceId;
	string m_strSubSystemVendorId;

	/////////////////////////////
	// Helper functions.
	/////////////////////////////

	void m_InitializeAsHostBridge();
	void m_EnumerateSecondaryBusFunctions();
	void m_MakeConfigSpaceCopy();
	void m_WriteIfChanged(DWORD dwHeaderOffset, UCHAR ucFieldSize);
	DWORD m_BarFromBarIndex(DWORD dwBarIndex);
	int m_BarIndexFromBarValue(DWORD dwBarValue);

	void m_ParsePciDevicesFile();
	void m_GetBarSizesFromRegistry();

	DWORD m_ReadConfigSpaceDword(ULONG ulBus, ULONG ulDevice, ULONG ulFunction, ULONG ulDwordOffset);
	BOOL m_ReadConfigSpaceHeader(ULONG ulBus, ULONG ulDevice, ULONG ulFunction, PTYPEX_HEADER pTypeXHeader);
	BOOL m_WriteConfigSpaceXxx(ULONG ulBus, ULONG ulDevice, ULONG ulFunction, ULONG ulByteOffset, UCHAR ucSize, ULONG ulData);
	DWORD m_ReadConfigSpaceXxx(ULONG ulBus, ULONG ulDevice, ULONG ulFunction, ULONG ulByteOffset, UCHAR ucSize);
};
