/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#pragma once

#include "PciFunctionView.h"
#include "AbstractTabPane.h"
#include "EditListCtrl.h"
#include <afxtempl.h>

class CPciFunctionView;

typedef enum HexEditMode {
	hemByte = 1, 
	hemWord = 2, 
	hemDword = 4
};

class CBarPane : public CAbstractTabPane
{
public:
	CBarPane(CPciFunctionView * pPciFunctionView, DWORD dwBarIndex);
	virtual ~CBarPane(void);

	//////////////////////////////////////////
	// Public interface.
	//////////////////////////////////////////

	// Functions defined by CAbstractBarPane we need to implement.
	void OnSize(UINT nType, int cx, int cy);
	void ActivatePane();
	void DeactivatePane();
	void ShowPciFunction(CPciFunction * pPciFunction = NULL);
	void SetHexadecimalDisplay(BOOL bHexadecimal);
	void OnValueEdited(int iItemEdited, int iSubItemEdited, char * pszNewValue);
	TABPANETYPE GetType(){ return BarPane; }
	void OnContextMenu(CWnd* pWnd, CPoint point);
	void OnRefresh();

	// Functions defined only for this class.
	void OnSignedDisplay();
	void On8bitDisplay();
	void On16bitDisplay();
	void On32bitDisplay();

	void OnGetDispInfo(LV_DISPINFO * pDispInfo);

private:

	//////////////////////////////////////////
	// Member variables.
	//////////////////////////////////////////

	// This is the pointer to our parent.
	CPciFunctionView * m_pPciFunctionView;

	// This is a special list control that
	// knows how to edit sub items.
	CEditListCtrl m_editListCtrl;

	// This is a cached pointer to the currently displayed CPciFunction object.
	// It is used so that we can refresh the view when toggling between hex and
	// decimal value display.
	CPciFunction * m_pCachedPciFunction;

	// This determines which of BAR0..BAR5 we are.
	DWORD m_dwBarIndex;

	// This determines if the values in the BAR panes
	// are displayed as hexadecimal or decimal.
	static BOOL m_bHexadecimal;	

	// This is the display mode (BYTE, WORD or DWORD) of the BAR panes.
	static HexEditMode m_HexEditMode; 

	// This is a cache that we use to store data 
	// read from the device's active BAR range.
	CMap<DWORD, DWORD, DWORD, DWORD> m_DataCache;


	//////////////////////////////////////////
	// Helper functions.
	//////////////////////////////////////////

	BOOL m_InitializeBarListCtrl();
	void m_UpdateBarEditListCtrl(CPciFunction * pPciFunction);
	void m_ConfigureDisplayMode();
	DWORD m_GetBarSize();
};
