/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#include "StdAfx.h"
#include "ConfigSpacePane.h"
#include "bcmenu.h"

CConfigSpacePane::CConfigSpacePane(CPciFunctionView * pPciFunctionView)
{
	// Save the pointer to our parent.
	m_pPciFunctionView = pPciFunctionView;

	// Make one-time initializations.
	m_InitializePciHeaderListCtrl();

	m_bHexadecimal = TRUE;
	m_pCachedPciFunction = NULL;
}

CConfigSpacePane::~CConfigSpacePane(void)
{

}

BOOL CConfigSpacePane::m_InitializePciHeaderListCtrl()
{
	// Subclass the list control defined in the dialog resources.
	BOOL bSuccess = m_editListCtrl.SubclassDlgItem(IDC_CONFIG_PANE_LIST_CTRL, m_pPciFunctionView);
	_ASSERTE(bSuccess);

	// Add columns to the list control.
	m_editListCtrl.InsertColumn(0, "Offset");
	m_editListCtrl.InsertColumn(1, "Size");
	m_editListCtrl.InsertColumn(2, "Name");
	m_editListCtrl.InsertColumn(3, "Value");
	m_editListCtrl.InsertColumn(4, "Decoded Information");

	// Set the column widths.
	m_editListCtrl.SetColumnWidth(0, 100);
	m_editListCtrl.SetColumnWidth(1, 100);
	m_editListCtrl.SetColumnWidth(2, 200);
	m_editListCtrl.SetColumnWidth(3, 150);
	m_editListCtrl.SetColumnWidth(4, 600);

	// Set the background color of the list control to the nice yellow...
	m_editListCtrl.SetBkColor(RGB(0xF7, 0xF3, 0xE9));

	// Configure various styles of the list control.
	m_editListCtrl.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES);

	return TRUE;
}

void CConfigSpacePane::m_UpdatePciHeaderListCtrl(CPciFunction * pPciFunction)
{
	// Clear all old items from the list control.
	CListCtrl * pListCtrl = (CListCtrl *) m_pPciFunctionView->GetDlgItem(IDC_CONFIG_PANE_LIST_CTRL);
	BOOL bSuccess = pListCtrl->DeleteAllItems();
	_ASSERTE(bSuccess);

	// If a multifunction device node is selected we
	// don't have data to output in the list control.
	if(pPciFunction == NULL)
		return;

	// There are two kinds of PCI Functions; bridges and non-bridges.
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	BOOL b64BitMemBar = FALSE;

	switch(pTypeXheader->Type0Hdr.ucHeaderType & 0x7f) // NOTE: Bit 7 indicates that the device is a multi-function device.
	{
		case 0:
			// Add fields from the Type 0 header.
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, usVendorID), sizeof(USHORT), "Vendor ID", pTypeXheader->Type0Hdr.usVendorID, pPciFunction->GetVendorIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, usDeviceID), sizeof(USHORT), "Device ID", pTypeXheader->Type0Hdr.usDeviceID, pPciFunction->GetDeviceIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, usCommandRegister), sizeof(USHORT), "Command Register", pTypeXheader->Type0Hdr.usCommandRegister, m_DecodeCommandRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, usStatusRegister), sizeof(USHORT), "Status Register", pTypeXheader->Type0Hdr.usStatusRegister, m_DecodeStatusRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucRevisionID), sizeof(UCHAR), "Revision ID", pTypeXheader->Type0Hdr.ucRevisionID);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucProgrammingInterface), sizeof(UCHAR), "Programming Interface", pTypeXheader->Type0Hdr.ucProgrammingInterface);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucSubClassCode), sizeof(UCHAR), "Subclass Code", pTypeXheader->Type0Hdr.ucSubClassCode, m_DecodeSubClassCodeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucClassCode), sizeof(UCHAR), "Class Code", pTypeXheader->Type0Hdr.ucClassCode, m_DecodeClassCodeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucCacheLineSize), sizeof(UCHAR), "Cache Line Size", pTypeXheader->Type0Hdr.ucCacheLineSize, m_DecodeCacheLineSizeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucLatencyTimer), sizeof(UCHAR), "Latency Timer", pTypeXheader->Type0Hdr.ucLatencyTimer, m_DecodeLatencyTimerRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucHeaderType), sizeof(UCHAR), "Header Type", pTypeXheader->Type0Hdr.ucHeaderType);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucBIST), sizeof(UCHAR), "BIST", pTypeXheader->Type0Hdr.ucBIST, m_DecodeBistRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulBAR0), sizeof(ULONG), "BAR 0", pTypeXheader->Type0Hdr.ulBAR0, m_DecodeBarRegister(pTypeXheader->Type0Hdr.ulBAR1, pTypeXheader->Type0Hdr.ulBAR0, pPciFunction->m_vdwBarSizes[0], &b64BitMemBar));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulBAR1), sizeof(ULONG), "BAR 1", pTypeXheader->Type0Hdr.ulBAR1, b64BitMemBar ? "" : m_DecodeBarRegister(0, pTypeXheader->Type0Hdr.ulBAR1, pPciFunction->m_vdwBarSizes[1], NULL));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulBAR2), sizeof(ULONG), "BAR 2", pTypeXheader->Type0Hdr.ulBAR2, m_DecodeBarRegister(pTypeXheader->Type0Hdr.ulBAR3, pTypeXheader->Type0Hdr.ulBAR2, pPciFunction->m_vdwBarSizes[2], &b64BitMemBar));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulBAR3), sizeof(ULONG), "BAR 3", pTypeXheader->Type0Hdr.ulBAR3, b64BitMemBar ? "" : m_DecodeBarRegister(0, pTypeXheader->Type0Hdr.ulBAR3, pPciFunction->m_vdwBarSizes[3], NULL));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulBAR4), sizeof(ULONG), "BAR 4", pTypeXheader->Type0Hdr.ulBAR4, m_DecodeBarRegister(pTypeXheader->Type0Hdr.ulBAR5, pTypeXheader->Type0Hdr.ulBAR4, pPciFunction->m_vdwBarSizes[4], &b64BitMemBar));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulBAR5), sizeof(ULONG), "BAR 5", pTypeXheader->Type0Hdr.ulBAR5, b64BitMemBar ? "" : m_DecodeBarRegister(0, pTypeXheader->Type0Hdr.ulBAR5, pPciFunction->m_vdwBarSizes[5], NULL));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulCardBusCisPointer), sizeof(ULONG), "Cardbus CIS Pointer", pTypeXheader->Type0Hdr.ulCardBusCisPointer);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, usSubSystemVendorID), sizeof(USHORT), "Subsystem Vendor ID", pTypeXheader->Type0Hdr.usSubSystemVendorID, pPciFunction->GetSubSystemVendorIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, usSubSystemDeviceID), sizeof(USHORT), "Subsystem Device ID", pTypeXheader->Type0Hdr.usSubSystemDeviceID);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulExpansionRomBAR), sizeof(ULONG), "Expansion ROM BAR", pTypeXheader->Type0Hdr.ulExpansionRomBAR);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucCapabilitiesPointer), sizeof(UCHAR), "Capabilities Pointer", pTypeXheader->Type0Hdr.ucCapabilitiesPointer);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, vucReserved1), sizeof(UCHAR), "Reserved", pTypeXheader->Type0Hdr.vucReserved1[0]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, vucReserved1) + 1, sizeof(UCHAR), "Reserved", pTypeXheader->Type0Hdr.vucReserved1[1]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, vucReserved1) + 2, sizeof(UCHAR), "Reserved", pTypeXheader->Type0Hdr.vucReserved1[2]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ulReserved2), sizeof(ULONG), "Reserved", pTypeXheader->Type0Hdr.ulReserved2);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucInterruptLine), sizeof(UCHAR), "Interrupt Line", pTypeXheader->Type0Hdr.ucInterruptLine, m_DecodeInterruptLineRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucInterruptPin), sizeof(UCHAR), "Interrupt Pin", pTypeXheader->Type0Hdr.ucInterruptPin, m_DecodeInterruptPinRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucMinGnt), sizeof(UCHAR), "Min Grant", pTypeXheader->Type0Hdr.ucMinGnt, m_DecodeMinGrantRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE0_HEADER, ucMaxLat), sizeof(UCHAR), "Max Latency", pTypeXheader->Type0Hdr.ucMaxLat, m_DecodeMaxLatencyRegister(pPciFunction));
			
			m_AddNewCapsInfoToPciHeaderListCtrl(pPciFunction, pListCtrl); 
			break;

		case 1:
			// Add fields from the Type 1 header.
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usVendorID), sizeof(USHORT), "Vendor ID", pTypeXheader->Type1Hdr.usVendorID, pPciFunction->GetVendorIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usDeviceID), sizeof(USHORT), "Device ID", pTypeXheader->Type1Hdr.usDeviceID, pPciFunction->GetDeviceIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usCommandRegister), sizeof(USHORT), "Command Register", pTypeXheader->Type1Hdr.usCommandRegister, m_DecodeCommandRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usStatusRegister), sizeof(USHORT), "Status Register", pTypeXheader->Type1Hdr.usStatusRegister, m_DecodeStatusRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucRevisionID), sizeof(UCHAR), "Revision ID", pTypeXheader->Type1Hdr.ucRevisionID);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucProgrammingInterface), sizeof(UCHAR), "Programming Interface", pTypeXheader->Type1Hdr.ucProgrammingInterface);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucSubClassCode), sizeof(UCHAR), "Subclass Code", pTypeXheader->Type1Hdr.ucSubClassCode, m_DecodeSubClassCodeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucClassCode), sizeof(UCHAR), "Class Code", pTypeXheader->Type1Hdr.ucClassCode, m_DecodeClassCodeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucCacheLineSize), sizeof(UCHAR), "Cache Line Size", pTypeXheader->Type1Hdr.ucCacheLineSize, m_DecodeCacheLineSizeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucLatencyTimer), sizeof(UCHAR), "Latency Timer", pTypeXheader->Type1Hdr.ucLatencyTimer, m_DecodeLatencyTimerRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucHeaderType), sizeof(UCHAR), "Header Type", pTypeXheader->Type1Hdr.ucHeaderType);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucBIST), sizeof(UCHAR), "BIST", pTypeXheader->Type1Hdr.ucBIST, m_DecodeBistRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ulBAR0), sizeof(ULONG), "BAR 0", pTypeXheader->Type1Hdr.ulBAR0, m_DecodeBarRegister(pTypeXheader->Type0Hdr.ulBAR1, pTypeXheader->Type0Hdr.ulBAR0, pPciFunction->m_vdwBarSizes[0], &b64BitMemBar));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ulBAR1), sizeof(ULONG), "BAR 1", pTypeXheader->Type1Hdr.ulBAR1, b64BitMemBar ? "" : m_DecodeBarRegister(0, pTypeXheader->Type0Hdr.ulBAR1, pPciFunction->m_vdwBarSizes[1], NULL));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucPrimaryBusNumber), sizeof(UCHAR), "Primary Bus Number", pTypeXheader->Type1Hdr.ucPrimaryBusNumber);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucSecondaryBusNumber), sizeof(UCHAR), "Secondary Bus Number", pTypeXheader->Type1Hdr.ucSecondaryBusNumber);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucSubordinateBusNumber), sizeof(UCHAR), "Subordinate Bus Number", pTypeXheader->Type1Hdr.ucSubordinateBusNumber);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucSecondaryLatencyTimer), sizeof(UCHAR), "Secondary Latency Timer", pTypeXheader->Type1Hdr.ucSecondaryLatencyTimer);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucIoBase), sizeof(UCHAR), "I/O Base", pTypeXheader->Type1Hdr.ucIoBase, m_DecodeIoRangeStart(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucIoLimit), sizeof(UCHAR), "I/O Limit", pTypeXheader->Type1Hdr.ucIoLimit, m_DecodeIoRangeEnd(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usSecondaryStatus), sizeof(USHORT), "Secondary Status", pTypeXheader->Type1Hdr.usSecondaryStatus);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usMemoryBase), sizeof(USHORT), "Memory Base", pTypeXheader->Type1Hdr.usMemoryBase, m_DecodeMemoryRangeStart(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usMemoryLimit), sizeof(USHORT), "Memory Limit", pTypeXheader->Type1Hdr.usMemoryLimit, m_DecodeMemoryRangeEnd(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usPrefetchableMemoryBase), sizeof(USHORT), "Prefetchable Memory Base", pTypeXheader->Type1Hdr.usPrefetchableMemoryBase, m_DecodePrefetchableMemoryRangeStart(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usPrefetchableMemoryLimit), sizeof(USHORT), "Prefetchable Memory Limit", pTypeXheader->Type1Hdr.usPrefetchableMemoryLimit, m_DecodePrefetchableMemoryRangeEnd(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ulPrefetchableMemoryBaseUpper32), sizeof(ULONG), "Prefetchable Memory Base Upper 32", pTypeXheader->Type1Hdr.ulPrefetchableMemoryBaseUpper32);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ulPrefetchableMemoryLimitUpper32), sizeof(ULONG), "Prefetchable Memory Limit Upper 32", pTypeXheader->Type1Hdr.ulPrefetchableMemoryLimitUpper32);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usIoBaseUpper16), sizeof(USHORT), "I/O Base Upper 16", pTypeXheader->Type1Hdr.usIoBaseUpper16);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usIoLimitUpper16), sizeof(USHORT), "I/O Limit Upper 16", pTypeXheader->Type1Hdr.usIoLimitUpper16);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucCapabilitiesPointer), sizeof(UCHAR), "Capabilities Pointer", pTypeXheader->Type1Hdr.ucCapabilitiesPointer);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, vucReserved1), sizeof(UCHAR), "Reserved", pTypeXheader->Type1Hdr.vucReserved1[0]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, vucReserved1) + 1, sizeof(UCHAR), "Reserved", pTypeXheader->Type1Hdr.vucReserved1[1]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, vucReserved1) + 2, sizeof(UCHAR), "Reserved", pTypeXheader->Type1Hdr.vucReserved1[2]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ulExpansionRomBAR), sizeof(ULONG), "Expansion ROM BAR", pTypeXheader->Type1Hdr.ulExpansionRomBAR);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucInterruptLine), sizeof(UCHAR), "Interrupt Line", pTypeXheader->Type1Hdr.ucInterruptLine, m_DecodeInterruptLineRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, ucInterruptPin), sizeof(UCHAR), "Interrupt Pin", pTypeXheader->Type1Hdr.ucInterruptPin, m_DecodeInterruptPinRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE1_HEADER, usBridgeControl), sizeof(USHORT), "Bridge Control", pTypeXheader->Type1Hdr.usBridgeControl);
			
			m_AddNewCapsInfoToPciHeaderListCtrl(pPciFunction, pListCtrl);
			break;

		case 2:
			// Add fields from the Type 2 header.
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usVendorID), sizeof(USHORT), "Vendor ID", pTypeXheader->Type2Hdr.usVendorID, pPciFunction->GetVendorIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usDeviceID), sizeof(USHORT), "Device ID", pTypeXheader->Type2Hdr.usDeviceID, pPciFunction->GetDeviceIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usCommandRegister), sizeof(USHORT), "Command Register", pTypeXheader->Type2Hdr.usCommandRegister, m_DecodeCommandRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usStatusRegister), sizeof(USHORT), "Status Register", pTypeXheader->Type2Hdr.usStatusRegister, m_DecodeStatusRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucRevisionID), sizeof(UCHAR), "Revision ID", pTypeXheader->Type2Hdr.ucRevisionID);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucProgrammingInterface), sizeof(UCHAR), "Programming Interface", pTypeXheader->Type2Hdr.ucProgrammingInterface);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucSubClassCode), sizeof(UCHAR), "Subclass Code", pTypeXheader->Type2Hdr.ucSubClassCode, m_DecodeSubClassCodeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucClassCode), sizeof(UCHAR), "Class Code", pTypeXheader->Type2Hdr.ucClassCode, m_DecodeClassCodeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucCacheLineSize), sizeof(UCHAR), "Cache Line Size", pTypeXheader->Type2Hdr.ucCacheLineSize, m_DecodeCacheLineSizeRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucLatencyTimer), sizeof(UCHAR), "Latency Timer", pTypeXheader->Type2Hdr.ucLatencyTimer, m_DecodeLatencyTimerRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucHeaderType), sizeof(UCHAR), "Header Type", pTypeXheader->Type2Hdr.ucHeaderType);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucBIST), sizeof(UCHAR), "BIST", pTypeXheader->Type2Hdr.ucBIST, m_DecodeBistRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ulCardbusSocketRegisters), sizeof(ULONG), "Cardbus Socket Registers", pTypeXheader->Type2Hdr.ulCardbusSocketRegisters);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, vucReserved), sizeof(UCHAR), "Reserved", pTypeXheader->Type2Hdr.vucReserved[0]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, vucReserved) + 1, sizeof(UCHAR), "Reserved", pTypeXheader->Type2Hdr.vucReserved[1]);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usSecondaryStatus), sizeof(USHORT), "Secondary Status", pTypeXheader->Type2Hdr.usSecondaryStatus);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucPciBusNumber), sizeof(UCHAR), "PCI Bus Number", pTypeXheader->Type2Hdr.ucPciBusNumber);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucCardBusBusNumber), sizeof(UCHAR), "Cardbus Bus Number", pTypeXheader->Type2Hdr.ucCardBusBusNumber);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucSubordinateBusNumber), sizeof(UCHAR), "Subordinate Bus Number", pTypeXheader->Type2Hdr.ucSubordinateBusNumber);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucCardbusLatencyTimer), sizeof(UCHAR), "Cardbus Latency Timer", pTypeXheader->Type2Hdr.ucCardbusLatencyTimer);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ulMemoryBaseRegister0), sizeof(ULONG), "Memory Base Register 0", pTypeXheader->Type2Hdr.ulMemoryBaseRegister0);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ulMemoryLimitRegister0), sizeof(ULONG), "Memory Limit Register 0", pTypeXheader->Type2Hdr.ulMemoryLimitRegister0);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ulMemoryBaseRegister1), sizeof(ULONG), "Memory Base Register 1", pTypeXheader->Type2Hdr.ulMemoryBaseRegister1);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ulMemoryLimitRegister1), sizeof(ULONG), "Memory Limit Register 1", pTypeXheader->Type2Hdr.ulMemoryLimitRegister1);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoBaseRegisterLower0), sizeof(ULONG), "I/O Base Register 0 Lower", pTypeXheader->Type2Hdr.usIoBaseRegisterLower0);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoBaseRegisterUpper0), sizeof(ULONG), "I/O Base Register 0 Upper", pTypeXheader->Type2Hdr.usIoBaseRegisterUpper0);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoLimitRegisterLower0), sizeof(ULONG), "I/O Limit Register 0 Lower", pTypeXheader->Type2Hdr.usIoLimitRegisterLower0);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoLimitRegisterUpper0), sizeof(ULONG), "I/O Limit Register 0 Upper", pTypeXheader->Type2Hdr.usIoLimitRegisterUpper0);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoBaseRegisterLower1), sizeof(ULONG), "I/O Base Register 1 Lower", pTypeXheader->Type2Hdr.usIoBaseRegisterLower1);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoBaseRegisterUpper1), sizeof(ULONG), "I/O Base Register 1 Upper", pTypeXheader->Type2Hdr.usIoBaseRegisterUpper1);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoLimitRegisterLower1), sizeof(ULONG), "I/O Limit Register 1 Lower", pTypeXheader->Type2Hdr.usIoLimitRegisterLower1);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usIoLimitRegisterUpper1), sizeof(ULONG), "I/O Limit Register 1 Upper", pTypeXheader->Type2Hdr.usIoLimitRegisterUpper1);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucInterruptLine), sizeof(UCHAR), "Interrupt Line", pTypeXheader->Type2Hdr.ucInterruptLine, m_DecodeInterruptLineRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ucInterruptPin), sizeof(UCHAR), "Interrupt Pin", pTypeXheader->Type2Hdr.ucInterruptPin, m_DecodeInterruptPinRegister(pPciFunction));
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usBridgeControl), sizeof(USHORT), "Bridge Control", pTypeXheader->Type2Hdr.usBridgeControl);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usSubSystemVendorID), sizeof(USHORT), "Subsystem Vendor ID", pTypeXheader->Type2Hdr.usSubSystemVendorID, pPciFunction->GetSubSystemVendorIdString().c_str());
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, usSubSystemDeviceID), sizeof(USHORT), "Subsystem Device ID", pTypeXheader->Type2Hdr.usSubSystemDeviceID);
			m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, ulLegacyModeBaseAddress), sizeof(ULONG), "Legacy Mode Base Address", pTypeXheader->Type2Hdr.ulLegacyModeBaseAddress);

			for(int i=0; i<14; i++)
			{
				m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, vulReserved) + 4*i, sizeof(ULONG), "Reserved", pTypeXheader->Type2Hdr.vulReserved[i]);
			}

			for(int i=0; i<32; i++)
			{
				m_AddItemToPciHeaderListCtrl(pListCtrl, offsetof(TYPE2_HEADER, vulVendorSpecific) + 4*i, sizeof(ULONG), "Vendor Specific", pTypeXheader->Type2Hdr.vulVendorSpecific[i]);
			}
	}
}

void CConfigSpacePane::m_AddItemToPciHeaderListCtrl(CListCtrl * pListCtrl, WORD wOffset, WORD wSize, char * szName, DWORD dwValue, CString strMeaning)
{
	// The offset is the main item, the size, name and value are the subitems.
	CString strTemp;
	
	// Set the Main 'offset' item.
	if(m_bHexadecimal)
		strTemp.Format("%02Xh", wOffset, wOffset);
	else
		strTemp.Format("%03ud", wOffset, wOffset);

	int iItemIndex = pListCtrl->InsertItem(pListCtrl->GetItemCount(), strTemp);

	// Set the subitems.
	switch(wSize)
	{
		case 1:
			pListCtrl->SetItemText(iItemIndex, 1, "BYTE");
			if(m_bHexadecimal)
				strTemp.Format("%02Xh", dwValue, dwValue);
			else
				strTemp.Format("%ud", dwValue, dwValue);
			break;

		case 2:
			pListCtrl->SetItemText(iItemIndex, 1, "WORD");
			if(m_bHexadecimal)
				strTemp.Format("%04Xh", dwValue, dwValue);
			else
				strTemp.Format("%ud", dwValue, dwValue);
			break;

		case 4:
			pListCtrl->SetItemText(iItemIndex, 1, "DWORD");
			if(m_bHexadecimal)
				strTemp.Format("%08Xh", dwValue, dwValue);
			else
				strTemp.Format("%ud", dwValue, dwValue);
			break;
	}

	pListCtrl->SetItemText(iItemIndex, 2, szName);	
	pListCtrl->SetItemText(iItemIndex, 3, strTemp);
	pListCtrl->SetItemText(iItemIndex, 4, strMeaning);

	// We hang the offset and size of the value off of the item so that 
	// we can retrieve exactly what value has been changed after the 
	// 'value' subitem has been edited.
	pListCtrl->SetItemData(iItemIndex, wOffset<<16 | wSize);

	pListCtrl->Update(iItemIndex);
}

CString CConfigSpacePane::m_DecodeCommandRegister(CPciFunction * pPciFunction)
{
	CString strCmdRegMeaning = "";
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	WORD wCommandRegister = pTypeXheader->Type0Hdr.usCommandRegister;
	
	if(wCommandRegister & (1<<9))
		strCmdRegMeaning += "Fast Back-to-back";

	if(wCommandRegister & (1<<8))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", SERR# Enable" : "SERR# Enable";

	if(wCommandRegister & (1<<7))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", Stepping Control" : "Stepping Control";

	if(wCommandRegister & (1<<6))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", Parity Error Response" : "Parity Error Response";

	if(wCommandRegister & (1<<5))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", VGA Palette Snoop" : "VGA Palette Snoop";

	if(wCommandRegister & (1<<4))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", Memory Write and Invalidate Enable" : "Memory Write and Invalidate Enable";

	if(wCommandRegister & (1<<3))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", Special Cycles" : "Special Cycles";

	if(wCommandRegister & (1<<2))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", Bus Master" : "Bus Master";

	if(wCommandRegister & (1<<1))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", Memory Space" : "Memory Space";

	if(wCommandRegister & (1<<0))
		strCmdRegMeaning += strCmdRegMeaning.GetLength() ? ", I/O Space" : "I/O Space";

	return strCmdRegMeaning;
}

CString CConfigSpacePane::m_DecodeStatusRegister(CPciFunction * pPciFunction)
{
	CString strStatusRegMeaning = "";
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	WORD wStatusRegister = pTypeXheader->Type0Hdr.usStatusRegister;
	
	if(wStatusRegister & (1<<15))
		strStatusRegMeaning += "Detected Parity Error";

	if(wStatusRegister & (1<<14))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Signaled System Error" : "Signaled System Error";

	if(wStatusRegister & (1<<13))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Received Master Abort" : "Received Master Abort";

	if(wStatusRegister & (1<<12))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Received Target Abort" : "Received Target Abort";

	if(wStatusRegister & (1<<11))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Signaled Target Abort" : "Signaled Target Abort";

	switch(wStatusRegister & (3<<9))
	{
		case 0:
			strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Fast Device Select Timing" : "Fast Device Select Timing";
			break;

		case 1:
			strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Medium Device Select Timing" : "Medium Device Select Timing";
			break;

		case 2:
			strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Slow Device Select Timing" : "Slow Device Select Timing";
			break;
	}

	if(wStatusRegister & (1<<8))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Master Data Parity Error" : "Master Data Parity Error";
	
	if(wStatusRegister & (1<<7))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Fast Back-to-Back Capable" : "Fast Back-to-Back Capable";

	if(wStatusRegister & (1<<6))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", User-Definable Features" : "User-Definable Features";

	if(wStatusRegister & (1<<5))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", 66 MHz Capable" : "66 MHz Capable";

	if(wStatusRegister & (1<<4))
		strStatusRegMeaning += strStatusRegMeaning.GetLength() ? ", Capabilities List Implemented" : "Capabilities List Implemented";

	return strStatusRegMeaning;
}

CString CConfigSpacePane::m_DecodeClassCodeRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucClassCode = pTypeXheader->Type0Hdr.ucClassCode;
	
	char * vszClasses[18] = {
		"(Unknown)",
		"Mass Storage Controller",
		"Network Controller",
		"Display Controller",
		"Multimedia Device",
		"Memory Controller",
		"Bridge Device",
		"Simple Communications Controller",
		"Base System Peripheral",
		"Input Device",
		"Docking Station",
		"Processor",
		"Serial Bus Controller",
		"Wireless Controller",
		"Intelligent I/O Controller",
		"Satellite Communications Controller",
		"Encryption/Decryption Controller",
		"Data Aquisition and Signal Processing Controller"
	};

	// Bounds check.
	if(ucClassCode == 0 || ucClassCode > 17)
		return "";

	return vszClasses[ucClassCode];
}

CString CConfigSpacePane::m_DecodeSubClassCodeRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucClassCode = pTypeXheader->Type0Hdr.ucClassCode;
	UCHAR ucSubClassCode = pTypeXheader->Type0Hdr.ucSubClassCode;
	UCHAR ucProgrammingInterface = pTypeXheader->Type0Hdr.ucProgrammingInterface;

	if(ucClassCode == 0 && ucSubClassCode == 1 && ucProgrammingInterface == 1)
		return "VGA-Compatible Device";

	if(ucClassCode == 1 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "SCSI Controller";

	if(ucClassCode == 1 && ucSubClassCode == 1)
		return "IDE Controller";

	if(ucClassCode == 1 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "Floppy Disk Controller";

	if(ucClassCode == 1 && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "IPI Controller";

	if(ucClassCode == 1 && ucSubClassCode == 4 && ucProgrammingInterface == 0)
		return "RAID Controller";

	if(ucClassCode == 1 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Mass-storage Controller";

	if(ucClassCode == 2 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Ethernet Controller";

	if(ucClassCode == 2 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "Token-ring Controller";

	if(ucClassCode == 2 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "FDDI Controller";

	if(ucClassCode == 2 && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "ATM Controller";

	if(ucClassCode == 2 && ucSubClassCode == 4 && ucProgrammingInterface == 0)
		return "ISDN Controller";
	
	if(ucClassCode == 2 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Network Controller";

	if(ucClassCode == 3 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "VGA-Compatible Controller";

	if(ucClassCode == 3 && ucSubClassCode == 0 && ucProgrammingInterface == 1)
		return "8514-Compatible Controller";

	if(ucClassCode == 3 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "XGA Controller";

	if(ucClassCode == 3 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "3D Controller";

	if(ucClassCode == 3 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Display Controller";

	if(ucClassCode == 4 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Video Device";

	if(ucClassCode == 4 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "Audio Device";

	if(ucClassCode == 4 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "Computer Telephony Device";

	if(ucClassCode == 4 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Multimedia Device";

	if(ucClassCode == 5 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "RAM Memory controller";

	if(ucClassCode == 5 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "Flash Memory controller";

	if(ucClassCode == 5 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Memory controller";

	if(ucClassCode == 6 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Host/PCI Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "PCI/ISA Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "PCI/EISA Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "PCI/Micro Channel Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 4 && ucProgrammingInterface == 0)
		return "PCI/PCI Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 4 && ucProgrammingInterface == 1)
		return "Subtractive-decode PCI/PCI Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 5 && ucProgrammingInterface == 0)
		return "PCI/PCMCIA Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 6 && ucProgrammingInterface == 0)
		return "PCI/NuBus Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 7 && ucProgrammingInterface == 0)
		return "PCI/CardBus Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 8)
		return "RACEWay Bridge";

	if(ucClassCode == 6 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Bridge";

	if(ucClassCode == 7 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Generic XT-Compatible Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 0 && ucProgrammingInterface == 1)
		return "16450-Compatible Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 0 && ucProgrammingInterface == 2)
		return "16550-Compatible Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 0 && ucProgrammingInterface == 3)
		return "16650-Compatible Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 0 && ucProgrammingInterface == 4)
		return "16750-Compatible Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 0 && ucProgrammingInterface == 5)
		return "16850-Compatible Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 0 && ucProgrammingInterface == 6)
		return "16950-Compatible Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "Parallel Port";

	if(ucClassCode == 7 && ucSubClassCode == 1 && ucProgrammingInterface == 1)
		return "Bi-directional Parallel Port";

	if(ucClassCode == 7 && ucSubClassCode == 1 && ucProgrammingInterface == 2)
		return "ECP 1.x-compliant Parallel Port";

	if(ucClassCode == 7 && ucSubClassCode == 1 && ucProgrammingInterface == 3)
		return "IEEE 1284 Controller";

	if(ucClassCode == 7 && ucSubClassCode == 1 && ucProgrammingInterface == 0xfe)
		return "IEEE 1284 Target Device";

	if(ucClassCode == 7 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "Multi-port Serial Controller";

	if(ucClassCode == 7 && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "Generic Modem";

	if(ucClassCode == 7 && ucSubClassCode == 3 && ucProgrammingInterface == 1)
		return "Hayes-Compatible Modem. 16450-Compatible Interface.";

	if(ucClassCode == 7 && ucSubClassCode == 3 && ucProgrammingInterface == 2)
		return "Hayes-Compatible Modem. 16550-Compatible Interface.";

	if(ucClassCode == 7 && ucSubClassCode == 3 && ucProgrammingInterface == 3)
		return "Hayes-Compatible Modem. 16650-Compatible Interface.";

	if(ucClassCode == 7 && ucSubClassCode == 3 && ucProgrammingInterface == 4)
		return "Hayes-Compatible Modem. 16750-Compatible Interface.";

	if(ucClassCode == 7 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Communications Device";

	if(ucClassCode == 8 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Generic 8259 Programmable Interrupt Controller";

	if(ucClassCode == 8 && ucSubClassCode == 0 && ucProgrammingInterface == 1)
		return "ISA Programmable Interrupt Controller";

	if(ucClassCode == 8 && ucSubClassCode == 0 && ucProgrammingInterface == 2)
		return "EISA Programmable Interrupt Controller";

	if(ucClassCode == 8 && ucSubClassCode == 0 && ucProgrammingInterface == 0x10)
		return "I/O APIC Interrupt Controller";

	if(ucClassCode == 8 && ucSubClassCode == 0 && ucProgrammingInterface == 0x20)
		return "I/O APIC Interrupt Controller";

	if(ucClassCode == 8 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "Generic 8237 DMA Controller";

	if(ucClassCode == 8 && ucSubClassCode == 1 && ucProgrammingInterface == 1)
		return "ISA DMA Controller";

	if(ucClassCode == 8 && ucSubClassCode == 1 && ucProgrammingInterface == 2)
		return "EISA DMA Controller";

	if(ucClassCode == 8 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "Generic 8254 Timer";

	if(ucClassCode == 8 && ucSubClassCode == 2 && ucProgrammingInterface == 1)
		return "ISA System Timer";

	if(ucClassCode == 8 && ucSubClassCode == 2 && ucProgrammingInterface == 2)
		return "EISA System Timer";

	if(ucClassCode == 8 && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "Generic RTC Controller";

	if(ucClassCode == 8 && ucSubClassCode == 3 && ucProgrammingInterface == 1)
		return "ISA RTC Controller";

	if(ucClassCode == 8 && ucSubClassCode == 4 && ucProgrammingInterface == 0)
		return "Generic PCI Hot-Plug Controller";

	if(ucClassCode == 8 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown System Peripheral";

	if(ucClassCode == 9 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Keyboard Controller";

	if(ucClassCode == 9 && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "Digitizer";

	if(ucClassCode == 9 && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "Mouse Controller";

	if(ucClassCode == 9 && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "Scanner Controller";

	if(ucClassCode == 9 && ucSubClassCode == 4 && ucProgrammingInterface == 0)
		return "Generic Game Port Controller";

	if(ucClassCode == 9 && ucSubClassCode == 4 && ucProgrammingInterface == 0x10)
		return "Game Port Controller";

	if(ucClassCode == 9 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Input Controller";

	if(ucClassCode == 0xa && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Generic Docking Station";

	if(ucClassCode == 0xa && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Docking Station";

	if(ucClassCode == 0xb && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "386 Processor";

	if(ucClassCode == 0xb && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "486 Processor";

	if(ucClassCode == 0xb && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "Pentium Processor";

	if(ucClassCode == 0xb && ucSubClassCode == 0x10 && ucProgrammingInterface == 0)
		return "Alpha Processor";

	if(ucClassCode == 0xb && ucSubClassCode == 0x20 && ucProgrammingInterface == 0)
		return "PowerPC Processor";

	if(ucClassCode == 0xb && ucSubClassCode == 0x30 && ucProgrammingInterface == 0)
		return "MIPS Processor";

	if(ucClassCode == 0xb && ucSubClassCode == 0x40 && ucProgrammingInterface == 0)
		return "Co-Processor";

	if(ucClassCode == 0xc && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "IEEE 1394 (Firewire)";

	if(ucClassCode == 0xc && ucSubClassCode == 0 && ucProgrammingInterface == 0x10)
		return "IEEE 1394 (Firewire) using OpenHCI Specification";

	if(ucClassCode == 0xc && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "ACCESS.bus";

	if(ucClassCode == 0xc && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "SSA (Serial Storage Architecture)";

	if(ucClassCode == 0xc && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "USB (Universal Serial Bus) Controller using UHCI (Universal Host Controller Interface)";

	if(ucClassCode == 0xc && ucSubClassCode == 3 && ucProgrammingInterface == 0x10)
		return "USB (Universal Serial Bus) Controller using OHCI (Open Host Controller Interface)";

	if(ucClassCode == 0xc && ucSubClassCode == 3 && ucProgrammingInterface == 0x80)
		return "USB (Universal Serial Bus) Controller using unknown programming interface";

	if(ucClassCode == 0xc && ucSubClassCode == 3 && ucProgrammingInterface == 0xfe)
		return "Unknown USB Device";

	if(ucClassCode == 0xc && ucSubClassCode == 4 && ucProgrammingInterface == 0)
		return "Fibre Channel Controller";

	if(ucClassCode == 0xc && ucSubClassCode == 5 && ucProgrammingInterface == 0)
		return "SMBus (System Management Bus) Controller";

	if(ucClassCode == 0xd && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "iRDA Compatible Controller";

	if(ucClassCode == 0xd && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "Consumer IR Controller";

	if(ucClassCode == 0xd && ucSubClassCode == 0x10 && ucProgrammingInterface == 0)
		return "RF Controller";

	if(ucClassCode == 0xd && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Wireless Controller";

	if(ucClassCode == 0xe && ucSubClassCode == 0)
		return "I2O (Intelligent I/O) Controller";

	if(ucClassCode == 0xf && ucSubClassCode == 1 && ucProgrammingInterface == 0)
		return "TV";

	if(ucClassCode == 0xf && ucSubClassCode == 2 && ucProgrammingInterface == 0)
		return "Audio";

	if(ucClassCode == 0xf && ucSubClassCode == 3 && ucProgrammingInterface == 0)
		return "Voice";

	if(ucClassCode == 0xf && ucSubClassCode == 4 && ucProgrammingInterface == 0)
		return "Data";

	if(ucClassCode == 0x10 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "Network and Computing Encryptor/Decryptor";

	if(ucClassCode == 0x10 && ucSubClassCode == 0x10 && ucProgrammingInterface == 0)
		return "Entertainment Encryptor/Decryptor";

	if(ucClassCode == 0x10 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Encryptor/Decryptor";

	if(ucClassCode == 0x11 && ucSubClassCode == 0 && ucProgrammingInterface == 0)
		return "DPIO Module";

	if(ucClassCode == 0x11 && ucSubClassCode == 0x80 && ucProgrammingInterface == 0)
		return "Unknown Data Aquisition and Signal Processing Controller";

	return "";
}

CString CConfigSpacePane::m_DecodeCacheLineSizeRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucCacheLineSize = pTypeXheader->Type0Hdr.ucCacheLineSize;	

	if(ucCacheLineSize == 0)
		return "";

	CString strCacheLineSize;
	strCacheLineSize.Format("%d bytes", ucCacheLineSize * 4);

	return strCacheLineSize;

}

CString CConfigSpacePane::m_DecodeLatencyTimerRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucLatencyTimer = pTypeXheader->Type0Hdr.ucLatencyTimer;	
	
	if(ucLatencyTimer == 0)
		return "";

	CString strLatencyTimerNs;
	strLatencyTimerNs.Format("%d ns guaranteed bus master ownership (%d PCI cycles at 33 MHz)", 
		(1000 * ucLatencyTimer) / 33, ucLatencyTimer);

	return strLatencyTimerNs;
}

CString CConfigSpacePane::m_DecodeBistRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucBIST = pTypeXheader->Type0Hdr.ucBIST;	
	
	if(ucBIST & 0x80)
		return "Built-in Self-test supported";

	return "";
}

CString CConfigSpacePane::m_DecodeBarRegister(DWORD dwBarHigh, DWORD dwBarLow, DWORD dwBarSize, BOOL * pb64BitMemBar)
{
	// NOTE: We cannot discover the range of the mem or I/O BARs without writing to them.
	// Since this might very well disturbe running PCI functions we don't report the range.
	// The user will optionally be able to probe the BARs when the BAR panes are activated.

	if(dwBarLow == 0)
		return "";

	BOOL bMemoryBar = (dwBarLow & (1<<0)) == 0;
	CString strMemBar = "";

	if(bMemoryBar)
	{
		UCHAR ucType = (UCHAR) ((dwBarLow>>1) & 3);
		BOOL b64BitMemBar = (ucType == 2);
		BOOL bPrefetchable = dwBarLow & (1<<3);
		DWORD dwBaseAddress = dwBarLow & 0xfffffff0;

		if(m_bHexadecimal)
		{
			if(b64BitMemBar)
				strMemBar.Format("64-bit %s Memory (Base = %08Xh:%08Xh, Size = %08Xh)", bPrefetchable ? "Prefetchable" : "", dwBarHigh, dwBaseAddress, dwBarSize);
			else
				strMemBar.Format("32-bit %s Memory (Base = %08Xh, Size = %08Xh)", bPrefetchable ? "Prefetchable" : "", dwBaseAddress, dwBarSize);
		}
		else
		{
			if(b64BitMemBar)
				strMemBar.Format("64-bit %s Memory (Base = %ud:%ud, Size = %ud)", bPrefetchable ? "Prefetchable" : "", dwBarHigh, dwBaseAddress, dwBarSize);
			else
				strMemBar.Format("32-bit %s Memory (Base = %ud, Size = %ud)", bPrefetchable ? "Prefetchable" : "", dwBaseAddress, dwBarSize);
		}

		// Set out-parameter.
		if(pb64BitMemBar)
		{
			*pb64BitMemBar = b64BitMemBar;
		}
	}
	else
	{
		DWORD dwBaseAddress = dwBarLow & 0xfffffffc;
		if(dwBaseAddress == 0)
			return "";

		if(m_bHexadecimal)
		{
			strMemBar.Format("I/O Range (Base = %08Xh, Size = %08Xh)", dwBaseAddress, dwBarSize);
		}
		else
		{
			strMemBar.Format("I/O Range (Base = %ud, Size = %ud)", dwBaseAddress, dwBarSize);
		}

		// Set out-parameter.
		if(pb64BitMemBar)
		{
			*pb64BitMemBar = FALSE;
		}
	}

	return strMemBar;	
}

CString CConfigSpacePane::m_DecodeInterruptPinRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucInterruptPin = pTypeXheader->Type0Hdr.ucInterruptPin;	
	
	switch(ucInterruptPin)
	{
		case 1:
			return "INTA#";
		case 2:
			return "INTB#";
		case 3:
			return "INTC#";
		case 4:
			return "INTD#";
		default:
			return "";
	}
}

CString CConfigSpacePane::m_DecodeInterruptLineRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucInterruptPin = pTypeXheader->Type0Hdr.ucInterruptPin;	
	UCHAR ucInterruptLine = pTypeXheader->Type0Hdr.ucInterruptLine;	

	CString strInterruptLine = "";

	if(ucInterruptPin)
	{
		strInterruptLine.Format("IRQ%d", ucInterruptLine);
	}
	
	return strInterruptLine;
}

CString CConfigSpacePane::m_DecodeMinGrantRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucMinGrant = pTypeXheader->Type0Hdr.ucMinGnt;	
	
	if(ucMinGrant == 0)
		return "";

	CString strMinGrantNs;
	strMinGrantNs.Format("Requests minimum %d ns bus master ownership (%d PCI cycles at 33 MHz)", 
		(1000 * ucMinGrant) / 33, ucMinGrant);

	return strMinGrantNs;
}

CString CConfigSpacePane::m_DecodeMaxLatencyRegister(CPciFunction * pPciFunction)
{
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	UCHAR ucMaxLatency = pTypeXheader->Type0Hdr.ucMaxLat;	
	
	if(ucMaxLatency == 0)
		return "";

	CString strucMaxLatencyNs;
	strucMaxLatencyNs.Format("Requests bus master ownership every %d ns interval (every %d PCI cycles at 33 MHz)", 
		(1000 * ucMaxLatency) / 33, ucMaxLatency);

	return strucMaxLatencyNs;
}

void CConfigSpacePane::OnValueEdited(int iItemEdited, int iSubItemEdited, char * pszNewValue)
{
	// The parameter indicates which row was clicked. We need to retrieve the 
	// PCI header offset and size from the edited item in the list control.
	DWORD dwData = (DWORD) m_editListCtrl.GetItemData(iItemEdited);
	WORD wPciHeaderValueOffset = (WORD) (dwData >> 16);
	WORD wPciHeaderValueSize = (WORD) (dwData & 0xffff);

	// Convert the new value into numeric form.
	ULONG ulEditedValue = strtoul(pszNewValue, NULL, m_bHexadecimal ? 16 : 10);

	// Get the header from the cached CPciFunctionobject.
	PTYPEX_HEADER pTypeXhdr = m_pCachedPciFunction->GetPciHeader();

	// Modify the PCI header.
	PUCHAR pucHeaderValue = (PUCHAR) (((char *)pTypeXhdr) + wPciHeaderValueOffset);
	PUSHORT pusHeaderValue = (PUSHORT) (((char *)pTypeXhdr) + wPciHeaderValueOffset);
	PULONG pulHeaderValue = (PULONG) (((char *)pTypeXhdr) + wPciHeaderValueOffset);
	
	switch(wPciHeaderValueSize)
	{
		case 1:
			*pucHeaderValue = (UCHAR) ulEditedValue;
			break;

		case 2:
			*pusHeaderValue = (USHORT) ulEditedValue;
			break;

		case 4:
			*pulHeaderValue = ulEditedValue;
			break;
	}

	// Write out the change to the device. This does a write followed by a read-back so we 
	// will know if the write 'took' simply by reading the changed header field again.
	m_pCachedPciFunction->WriteChanges();

	// Get the resulting value.
	ULONG ulResultingValue;

	switch(wPciHeaderValueSize)
	{
		case 1:
			ulResultingValue = *pucHeaderValue;
			break;

		case 2:
			ulResultingValue = *pusHeaderValue;
			break;

		case 4:
			ulResultingValue = *pulHeaderValue;
			break;
	}

	// Indicate via sound if the value didn't "take".
	if(ulResultingValue != ulEditedValue)
	{
		MessageBeep(MB_ICONEXCLAMATION);
	}

	// Format and display the resulting value in right radix.
	CString strResultingValue;

	switch(wPciHeaderValueSize)
	{
		case 1:
			if(m_bHexadecimal)
				strResultingValue.Format("%02Xh", ulResultingValue);
			else
				strResultingValue.Format("%ud", ulResultingValue);
			break;

		case 2:
			if(m_bHexadecimal)
				strResultingValue.Format("%04Xh", ulResultingValue);
			else
				strResultingValue.Format("%ud", ulResultingValue);
			break;

		case 4:
			if(m_bHexadecimal)
				strResultingValue.Format("%08Xh", ulResultingValue);
			else
				strResultingValue.Format("%ud", ulResultingValue);
			break;
	}

	// Set the resulting value in the 'value' column.
	m_editListCtrl.SetItemText(iItemEdited, iSubItemEdited, strResultingValue);
}

void CConfigSpacePane::m_AddNewCapsInfoToPciHeaderListCtrl(CPciFunction * pPciFunction, CListCtrl * pListCtrl)
{
	// We need to extract all New Capabilities entres from the New Capabilities 
	// List and insert NEW_CAPS_ENTRY structure pointers into a CPtrArray. Once
	// identified we need to sort the CPtrArray so that the New Capabilities are
	// sorted in the order they reside in the PCI configuration address space.
	// We need to have the New Caps entries sorted so that we insert them in the
	// correct order into the list control.
	CPtrArray ptrArray;

	// Get the first New Caps entry and the DWORD offset it resides at in the config space. 
	DWORD dwDwordOffset;
	PCAPABILITIES_HEADER pCapsHdr = pPciFunction->GetFirstCapabilitiesHeader(&dwDwordOffset);

	// Insert CAPABILITIES_HEADER pointers until we reach the end of the list.
	while(pCapsHdr)
	{
		// Insert the current New Caps Entry.
		PNEW_CAPS_ENTRY pNewCapsEntry = new NEW_CAPS_ENTRY;
		pNewCapsEntry->dwStartDwordOffset = dwDwordOffset;
		pNewCapsEntry->pCapsHdr = pCapsHdr;

		ptrArray.Add(pNewCapsEntry);
		
		// Get the next CAPABILITIES_HEADER pointer.
		pCapsHdr = pPciFunction->GetNextCapabilitiesHeader(pCapsHdr, &dwDwordOffset);
	}

	// Insert the New Caps information and the surrounding DWORDs.
	PTYPEX_HEADER pTypeXheader = pPciFunction->GetPciHeader();
	for(DWORD dwDwordIndex=16; dwDwordIndex<64; dwDwordIndex++)
	{
		m_AddItemToPciHeaderListCtrl(pListCtrl, dwDwordIndex*4, sizeof(DWORD), "Vendor Specific",  pTypeXheader->vulConfig[dwDwordIndex], m_DecodeNewCapsRegister(dwDwordIndex, &ptrArray));		
	}

	// Delete all NEW_CAPS_ENTRY structures.
	for(int iIndex=0; iIndex < ptrArray.GetSize(); iIndex++)
	{
		delete ptrArray[iIndex];
	}
}

CString CConfigSpacePane::m_DecodeNewCapsRegister(DWORD dwDwordOffset, CPtrArray * pPtrArray)
{
	// Search to see if there is any NEW_CAPS_ENTRY in the CPtrArray that matches the DWORD index.
	PNEW_CAPS_ENTRY pNewCapsEntry = NULL;
	for(int iIndex=0; iIndex < pPtrArray->GetSize(); iIndex++)
	{
		// Get the NEW_CAPS_ENTRY pointer from the array.
		pNewCapsEntry = (PNEW_CAPS_ENTRY) pPtrArray->GetAt(iIndex);
		if(pNewCapsEntry->dwStartDwordOffset == dwDwordOffset)
		{
			// Decode the type of the New Capability Entry.
			CString strDecodedNewCaps;
			switch(pNewCapsEntry->pCapsHdr->ucCapabilityID)
			{
				case CID_PCI_POWER_MANAGEMENT:
					{
						PCAPABILITY_PCI_POWER_MANAGEMENT pCapsPciPowerManagement = (PCAPABILITY_PCI_POWER_MANAGEMENT) pNewCapsEntry->pCapsHdr;
						
						if(m_bHexadecimal)
						{
							strDecodedNewCaps.Format("New Capability: PCI Power Management ("
								"Power Management Capabilities = %04Xh, Control/Status Register = %04Xh, "
								"Bridge Support Extensions = %02Xh, Data Register = %02Xh)",
								pCapsPciPowerManagement->wPowerManagementCapabilities,
								pCapsPciPowerManagement->wControlStatusRegister,
								pCapsPciPowerManagement->ucBridgeSupportExtensions,
								pCapsPciPowerManagement->ucDataRegister);
						}
						else
						{
							strDecodedNewCaps.Format("New Capability: PCI Power Management ("
								"Power Management Capabilities = %ud, Control/Status Register = %ud, "
								"Bridge Support Extensions = %ud, Data Register = %ud)",
								pCapsPciPowerManagement->wPowerManagementCapabilities,
								pCapsPciPowerManagement->wControlStatusRegister,
								pCapsPciPowerManagement->ucBridgeSupportExtensions,
								pCapsPciPowerManagement->ucDataRegister);
						}
						return strDecodedNewCaps;
					}
					break;

				case CID_AGP:
					{
						PCAPABILITY_AGP pCapsAgp = (PCAPABILITY_AGP) pNewCapsEntry->pCapsHdr;

						if(m_bHexadecimal)
						{
							strDecodedNewCaps.Format("New Capability: AGP ("
								"AGP Status Register = %08Xh, AGP Command Register = %08Xh)",
								pCapsAgp->dwAgpStatusRegister,
								pCapsAgp->dwAgpCommandRegister);
						}
						else
						{
							strDecodedNewCaps.Format("New Capability: AGP ("
								"AGP Status Register = %ud, AGP Command Register = %ud)",
								pCapsAgp->dwAgpStatusRegister,
								pCapsAgp->dwAgpCommandRegister);
						}

						return strDecodedNewCaps;
					}
					break;

				case CID_VPD:
					{
						PCAPABILITY_VPD pCapsVpd = (PCAPABILITY_VPD) pNewCapsEntry->pCapsHdr;
						
						if(m_bHexadecimal)
						{
							strDecodedNewCaps.Format("New Capability: VPD ("
								"VPD Address Register = %04Xh, VPD Data Register = %08Xh)",
								pCapsVpd->wVpdAddressRegister,
								pCapsVpd->dwVpdDataRegister);
						}
						else
						{
							strDecodedNewCaps.Format("New Capability: VPD ("
									"VPD Address Register = %ud, VPD Data Register = %ud)",
									pCapsVpd->wVpdAddressRegister,
									pCapsVpd->dwVpdDataRegister);
						}

						return strDecodedNewCaps;
					}
					break;			
			
				case CID_SLOT_IDENTIFICATION:
					{
						PCAPABILITY_SLOT_IDENTIFICATION pCapsSlotId = (PCAPABILITY_SLOT_IDENTIFICATION) pNewCapsEntry->pCapsHdr;

						if(m_bHexadecimal)
						{
							strDecodedNewCaps.Format("New Capability: Slot Identification ("
								"Expansion Slot Register = %02Xh, Chassis Number = %02Xh)",
								pCapsSlotId->ucExpansionSlotRegister,
								pCapsSlotId->ucChassisNumber);
						}
						else
						{
							strDecodedNewCaps.Format("New Capability: Slot Identification ("
								"Expansion Slot Register = %ud, Chassis Number = %ud)",
								pCapsSlotId->ucExpansionSlotRegister,
								pCapsSlotId->ucChassisNumber);
						}

						return strDecodedNewCaps;
					}
					break;

				case CID_MSI:
					{
						PCAPABILITY_MSI pCapsMsi = (PCAPABILITY_MSI) pNewCapsEntry->pCapsHdr;
					
						if(m_bHexadecimal)
						{
							// Hex representation.
							if(pCapsMsi->u.MsiCapsRegs32.wMessageControlRegister & (1<<7))
							{
								// 64-bit MSI structure.
								strDecodedNewCaps.Format("New Capability: MSI ("
									"Message Control Register = %04Xh, Lower Message Address Register = %08Xh, "
									"Upper Message Address Register = %08Xh, Message Data Register = %04Xh)",
										pCapsMsi->u.MsiCapsRegs64.wMessageControlRegister,
										pCapsMsi->u.MsiCapsRegs64.dwMessageAddressRegisterLo,
										pCapsMsi->u.MsiCapsRegs64.dwMessageAddressRegisterHi,
										pCapsMsi->u.MsiCapsRegs64.wMessageDataRegister);

							}
							else
							{
								// 32-bit MSI structure.
								strDecodedNewCaps.Format("New Capability: MSI ("
									"Message Control Register = %04Xh, Message Address Register = %08Xh, "
									"Message Data Register = %04Xh)",
										pCapsMsi->u.MsiCapsRegs32.wMessageControlRegister,
										pCapsMsi->u.MsiCapsRegs32.dwMessageAddressRegister,
										pCapsMsi->u.MsiCapsRegs32.wMessageDataRegister);
							}
						}
						else
						{
							// Decimal representation.
							if(pCapsMsi->u.MsiCapsRegs32.wMessageControlRegister & (1<<7))
							{
								// 64-bit MSI structure.
								strDecodedNewCaps.Format("New Capability: MSI ("
									"Message Control Register = %ud, Lower Message Address Register = %ud, "
									"Upper Message Address Register = %ud, Message Data Register = %ud)",
										pCapsMsi->u.MsiCapsRegs64.wMessageControlRegister,
										pCapsMsi->u.MsiCapsRegs64.dwMessageAddressRegisterLo,
										pCapsMsi->u.MsiCapsRegs64.dwMessageAddressRegisterHi,
										pCapsMsi->u.MsiCapsRegs64.wMessageDataRegister);

							}
							else
							{
								// 32-bit MSI structure.
								strDecodedNewCaps.Format("New Capability: MSI ("
									"Message Control Register = %ud, Message Address Register = %ud, "
									"Message Data Register = %ud)",
										pCapsMsi->u.MsiCapsRegs32.wMessageControlRegister,
										pCapsMsi->u.MsiCapsRegs32.dwMessageAddressRegister,
										pCapsMsi->u.MsiCapsRegs32.wMessageDataRegister);
							}
						}

						return strDecodedNewCaps;
					}
					break;

				case CID_CPCI_HOT_SWAP:
					return "New Capability: CompactPCI Hot Swap";
					break;

				case CID_PCIX:
					return "New Capability: PCI-X";
					break;

				case CID_AMD:
					return "New Capability: Reserved for AMD";
					break;

				case CID_VENDOR_SPECIFIC:
					return "New Capability: Vendor Specific";
					break;

				case CID_DEBUG_PORT:
					return "New Capability: Debug Port";
					break;

				case CID_CPCI_CENTRAL_RESOURCE_CONTROL:
					return "New Capability: CompactPCI Central Resource Control";
					break;		
			
				case CID_PCI_HOT_PLUG:
					return "New Capability: PCI Hot plug";
					break;	
			}

		}
	}

	return "";
}

void CConfigSpacePane::OnSize(UINT nType, int cx, int cy)
{
	////////////////////////////////////////////////////////////////////
	//	Resize the list control width and height so that 
	//	it covers the whole area under the tab control.
	////////////////////////////////////////////////////////////////////

	// Get the current position/size of the tab control. 
	CTabCtrl * pTabCtrl = (CTabCtrl *) m_pPciFunctionView->GetDlgItem(IDC_TAB);
	CRect rcTabCtrl;
	pTabCtrl->GetWindowRect(&rcTabCtrl);

	CListCtrl * pListCtrl = (CListCtrl *) m_pPciFunctionView->GetDlgItem(IDC_CONFIG_PANE_LIST_CTRL);
	if(pListCtrl)
	{
		// Calculate the new height of the list control.
		// This is the client area minus the height of the tab control.
		CRect rcClient;
		m_pPciFunctionView->GetClientRect(&rcClient);
		int iTabCtrlHeight = rcTabCtrl.Height();
		int iListCtrlHeight = rcClient.Height() - iTabCtrlHeight;

		// Set the new width and height of the list control.
		pListCtrl->SetWindowPos(m_pPciFunctionView, 0 , iTabCtrlHeight, cx, iListCtrlHeight, SWP_NOOWNERZORDER | SWP_NOZORDER);
	}
}

void CConfigSpacePane::ActivatePane()
{
	// We activate the window by making it highest in the Z-order.
	CWnd * pWnd = m_pPciFunctionView->GetDlgItem(IDC_CONFIG_PANE_LIST_CTRL);
	pWnd->SetWindowPos(&CWnd::wndTop, 0 , 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	pWnd->ShowWindow(SW_SHOW);
}

void CConfigSpacePane::DeactivatePane()
{
	// We deactivate the window by hiding it.
	CWnd * pWnd = m_pPciFunctionView->GetDlgItem(IDC_CONFIG_PANE_LIST_CTRL);
	pWnd->ShowWindow(SW_HIDE);
}

void CConfigSpacePane::ShowPciFunction(CPciFunction * pPciFunction)
{
	// Cache the CPciFunction object so we can refresh the view 
	// when switching between hex and decimal value display.
	m_pCachedPciFunction = pPciFunction;

	// Update the PCI Header list control.
	m_UpdatePciHeaderListCtrl(m_pCachedPciFunction);
}

void CConfigSpacePane::SetHexadecimalDisplay(BOOL bHexadecimal)
{ 
	// Set the new radix to use in the list control.
	m_bHexadecimal = bHexadecimal; 

	// Update the PCI Header list control.
	m_UpdatePciHeaderListCtrl(m_pCachedPciFunction);
}

CString CConfigSpacePane::m_DecodeIoRangeStart(CPciFunction * pPciFunction)
{
	CString strDecodedIoRangeStart;

	// Get the I/O Base register.
	PTYPE1_HEADER pType1Header = (PTYPE1_HEADER) pPciFunction->GetPciHeader();
	USHORT usIoBaseLow = (pType1Header->ucIoBase & 0xf0) << 8;

	// Is this a 32-bit I/O address decode?
	if(pType1Header->ucIoBase & 0x0f)
	{
		// 32-bit I/O address decode.

		// Get the upper 16 bits of the base.
		USHORT usIoBaseUpper16 = pType1Header->usIoBaseUpper16;
		
		// Calculate the 32-bit start address.
		DWORD dwStartAddress = usIoBaseUpper16 << 16 | usIoBaseLow;
		
		// Format the 32-bit I/O address range start.
		strDecodedIoRangeStart.Format("Secondary Bus 32-bit I/O Address Range Start: %08Xh", dwStartAddress);
	}
	else
	{
		// 16-bit I/O address decode.

		// Format the 16-bit I/O address range start.
		strDecodedIoRangeStart.Format("Secondary Bus 16-bit I/O Address Range Start: %04Xh", usIoBaseLow);
	}

	// All ok.
	return strDecodedIoRangeStart;
}

CString CConfigSpacePane::m_DecodeIoRangeEnd(CPciFunction * pPciFunction)
{
	CString strDecodedIoRangeEnd;

	// Get the I/O Limit register.
	PTYPE1_HEADER pType1Header = (PTYPE1_HEADER) pPciFunction->GetPciHeader();
	USHORT usIoLimitLow = (pType1Header->ucIoLimit & 0xf0) << 8  | 0xfff;

	// Is this a 32-bit I/O address decode?
	if(pType1Header->ucIoLimit & 0x0f)
	{
		// 32-bit I/O address decode.

		// Get the upper 16 bits of the limit.
		USHORT usIoLimitUpper16 = pType1Header->usIoLimitUpper16;
		
		// Calculate the 32-bit end address.
		DWORD dwEndAddress = usIoLimitUpper16 << 16 | usIoLimitLow;
		
		// Format the 32-bit I/O address range end.
		strDecodedIoRangeEnd.Format("Secondary Bus 32-bit I/O Address Range End: %08Xh", dwEndAddress);
	}
	else
	{
		// 16-bit I/O address decode.

		// Format the 16-bit I/O address range end.
		strDecodedIoRangeEnd.Format("Secondary Bus 16-bit I/O Address Range End: %04Xh", usIoLimitLow);
	}

	// All ok.
	return strDecodedIoRangeEnd;
}

CString CConfigSpacePane::m_DecodeMemoryRangeStart(CPciFunction * pPciFunction)
{
	CString strDecodedMemoryRangeStart;

	// Get the Memory Base register.
	PTYPE1_HEADER pType1Header = (PTYPE1_HEADER) pPciFunction->GetPciHeader();
	DWORD dwMemoryBase = (pType1Header->usMemoryBase & 0xfff0) << 16;

	// Format the 32-bit memory address range start.
	strDecodedMemoryRangeStart.Format("Secondary Bus Memory-mapped 32-bit I/O Address Range Start: %08Xh", dwMemoryBase);

	// All ok.
	return strDecodedMemoryRangeStart;
}

CString CConfigSpacePane::m_DecodeMemoryRangeEnd(CPciFunction * pPciFunction)
{
	CString strDecodedMemoryRangeEnd;

	// Get the Memory Limit register.
	PTYPE1_HEADER pType1Header = (PTYPE1_HEADER) pPciFunction->GetPciHeader();
	DWORD dwMemoryLimit = (pType1Header->usMemoryLimit & 0xfff0) << 16 | 0xfffff;

	// Format the 32-bit memory address range end.
	strDecodedMemoryRangeEnd.Format("Secondary Bus Memory-mapped 32-bit I/O Address Range End: %08Xh", dwMemoryLimit);

	// All ok.
	return strDecodedMemoryRangeEnd;
}

CString CConfigSpacePane::m_DecodePrefetchableMemoryRangeStart(CPciFunction * pPciFunction)
{
	CString strDecodedPrefetchableMemoryRangeStart;

	// Get the Base register.
	PTYPE1_HEADER pType1Header = (PTYPE1_HEADER) pPciFunction->GetPciHeader();
	DWORD dwPrefetchableMemoryBase = (pType1Header->usPrefetchableMemoryBase & 0xfff0) << 16;

	// Is this a 64-bit address decode?
	if(pType1Header->usPrefetchableMemoryBase & 0xf)
	{
		// 64-bit address decode.

		// Get the upper 32 bits of the base.
		DWORD dwPrefetchableMemoryBaseUpper32 = pType1Header->ulPrefetchableMemoryBaseUpper32;
		
		// Format the 64-bit address range start.
		strDecodedPrefetchableMemoryRangeStart.Format("Secondary Bus 64-bit Prefetchable Memory Address Range Start: %08Xh:%08Xh", 
			dwPrefetchableMemoryBaseUpper32, dwPrefetchableMemoryBase);
	}
	else
	{
		// 32-bit address decode.

		// Format the 16-bit address range start.
		strDecodedPrefetchableMemoryRangeStart.Format("Secondary Bus 32-bit Prefetchable Memory Address Range Start: %08Xh", dwPrefetchableMemoryBase);
	}

	// All ok.
	return strDecodedPrefetchableMemoryRangeStart;
}

CString CConfigSpacePane::m_DecodePrefetchableMemoryRangeEnd(CPciFunction * pPciFunction)
{
	CString strDecodedPrefetchableMemoryRangeEnd;

	// Get the Limit register.
	PTYPE1_HEADER pType1Header = (PTYPE1_HEADER) pPciFunction->GetPciHeader();
	DWORD dwPrefetchableMemoryLimit = (pType1Header->usPrefetchableMemoryLimit & 0xfff0) << 16 | 0xfffff;

	// Is this a 64-bit address decode?
	if(pType1Header->usPrefetchableMemoryLimit & 0xf)
	{
		// 64-bit address decode.

		// Get the upper 32 bits of the limit.
		DWORD dwPrefetchableMemoryLimitUpper32 = pType1Header->ulPrefetchableMemoryLimitUpper32;
		
		// Format the 64-bit address range end.
		strDecodedPrefetchableMemoryRangeEnd.Format("Secondary Bus 64-bit Prefetchable Memory Address Range End: %08Xh:%08Xh", 
			dwPrefetchableMemoryLimitUpper32, dwPrefetchableMemoryLimit);
	}
	else
	{
		// 32-bit address decode.

		// Format the 16-bit address range end.
		strDecodedPrefetchableMemoryRangeEnd.Format("Secondary Bus 32-bit Prefetchable Memory Address Range End: %08Xh", dwPrefetchableMemoryLimit);
	}

	// All ok.
	return strDecodedPrefetchableMemoryRangeEnd;
}

void CConfigSpacePane::OnContextMenu(CWnd* pWnd, CPoint point)
{
	// The user has right-clicked in the view while our 
	// pane was active so we display our context menu.
	BCMenu menu;
	menu.LoadMenu(IDR_CONTEXTMENU_CONFIGSPACEPANE);
	CMenu * pContextMenu = menu.GetSubMenu(0);
	if(m_bHexadecimal)
	{
		pContextMenu->CheckMenuItem(IDM_HEXADECIMAL, MF_BYCOMMAND | MF_CHECKED);
	}

	pContextMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, point.x, point.y, m_pPciFunctionView);
}

void CConfigSpacePane::OnRefresh()
{
	
}