/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#pragma once

#include "PciFunctionView.h"
#include "AbstractTabPane.h"
#include "EditListCtrl.h"

class CPciFunctionView;

class CConfigSpacePane : public CAbstractTabPane
{
public:
	CConfigSpacePane(CPciFunctionView * pPciFunctionView);
	~CConfigSpacePane(void);

	//////////////////////////////////////////
	// Public interface.
	//////////////////////////////////////////

	void OnSize(UINT nType, int cx, int cy);
	void ActivatePane();
	void DeactivatePane();
	void ShowPciFunction(CPciFunction * pPciFunction = NULL);
	void SetHexadecimalDisplay(BOOL bHexadecimal);
	void OnValueEdited(int iItemEdited, int iSubItemEdited, char * pszNewValue);
	TABPANETYPE GetType(){ return ConfigSpacePane; }
	void OnContextMenu(CWnd* pWnd, CPoint point);
	void OnRefresh();

private:

	//////////////////////////////////////////
	// Member variables.
	//////////////////////////////////////////

	// This is the pointer to our parent.
	CPciFunctionView * m_pPciFunctionView;

	// This is a special list control that
	// knows how to edit sub items.
	CEditListCtrl m_editListCtrl;

	// This determines if the values in the view 
	// are displayed as hexadecimal or decimal.
	BOOL m_bHexadecimal;

	// This is a cached pointer to the currently displayed CPciFunction object.
	// It is used so that we can refresh the view when toggling between hex and
	// decimal value display.
	CPciFunction * m_pCachedPciFunction;

	//////////////////////////////////////////
	// Helper functions.
	//////////////////////////////////////////

	BOOL m_InitializePciHeaderListCtrl();
	void m_AddItemToPciHeaderListCtrl(CListCtrl * pListCtrl, WORD wOffset, WORD wSize, char * szName, DWORD dwValue, CString strMeaning = "");
	void m_UpdatePciHeaderListCtrl(CPciFunction * pPciFunction);

	CString m_DecodeCommandRegister(CPciFunction * pPciFunction);
	CString m_DecodeStatusRegister(CPciFunction * pPciFunction);
	CString m_DecodeClassCodeRegister(CPciFunction * pPciFunction);
	CString m_DecodeSubClassCodeRegister(CPciFunction * pPciFunction);
	CString m_DecodeCacheLineSizeRegister(CPciFunction * pPciFunction);
	CString m_DecodeLatencyTimerRegister(CPciFunction * pPciFunction);
	CString m_DecodeBistRegister(CPciFunction * pPciFunction);
	CString m_DecodeBarRegister(DWORD dwBarHigh, DWORD dwBarLow, DWORD dwBarSize, BOOL * pb64BitMemBar);
	CString m_DecodeInterruptPinRegister(CPciFunction * pPciFunction);
	CString m_DecodeInterruptLineRegister(CPciFunction * pPciFunction);
	CString m_DecodeMinGrantRegister(CPciFunction * pPciFunction);
	CString m_DecodeMaxLatencyRegister(CPciFunction * pPciFunction);
	CString m_DecodeNewCapsRegister(DWORD dwDwordIndex, CPtrArray * pPtrArray);

	CString m_DecodeIoRangeStart(CPciFunction * pPciFunction);
	CString m_DecodeIoRangeEnd(CPciFunction * pPciFunction);
	CString m_DecodeMemoryRangeStart(CPciFunction * pPciFunction);
	CString m_DecodeMemoryRangeEnd(CPciFunction * pPciFunction);

	CString m_DecodePrefetchableMemoryRangeStart(CPciFunction * pPciFunction);
	CString m_DecodePrefetchableMemoryRangeEnd(CPciFunction * pPciFunction);

	void m_AddNewCapsInfoToPciHeaderListCtrl(CPciFunction * pPciFunction, CListCtrl * pListCtrl);
};
