/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#include "stdafx.h"
#include "PciExplorer.h"
#include "CustomTabCtrl.h"
#include ".\customtabctrl.h"


// CCustomTabCtrl

IMPLEMENT_DYNAMIC(CCustomTabCtrl, CTabCtrl)
CCustomTabCtrl::CCustomTabCtrl()
{
}

CCustomTabCtrl::~CCustomTabCtrl()
{
}


BEGIN_MESSAGE_MAP(CCustomTabCtrl, CTabCtrl)
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
END_MESSAGE_MAP()

BOOL CCustomTabCtrl::OnEraseBkgnd(CDC* pDC)
{
	// Set our background color to the nice yellow 
	// shade Visual Studio .NET uses for its tabs.
	CRect rcTabCtrl; 
    GetClientRect (&rcTabCtrl); 
    pDC->FillSolidRect(&rcTabCtrl, RGB(0xF7, 0xF3, 0xE9)); 

	// Make the area under the tabs gray to mimic the way of the .NET tabs.
	CRect rcTab;
	BOOL bSuccess = GetItemRect(0, rcTab);
	CRect rcClient(0, rcTab.Height()+2, rcTabCtrl.Width(), rcTabCtrl.Height());
	pDC->FillSolidRect(&rcClient, RGB(0xD4, 0xD0, 0xC8)); 

	// TRUE: Keeps GDI from drawing the background.
	return TRUE;
}


void CCustomTabCtrl::PreSubclassWindow()
{
	// Make the tabs ownerdrawn.
	BOOL bSuccess = ModifyStyle(0, TCS_OWNERDRAWFIXED | TCS_FOCUSNEVER );
}

void CCustomTabCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	int iSelectedTab = GetCurSel();
	int iCurrentTab = lpDrawItemStruct->itemID;
	BOOL bBoldFont = (iSelectedTab == iCurrentTab);
	CString strTabText;
		
	// Draw the content of the tabs. We need to do this ourselves in 
	// order to have the right background color and look of the tabs.
	CDC * dc = CDC::FromHandle(lpDrawItemStruct->hDC);
	CBrush brush(iCurrentTab == iSelectedTab ? RGB(0xD4, 0xD0, 0xC8) : RGB(0xF7, 0xF3, 0xE9));
	CRect rcDraw(&lpDrawItemStruct->rcItem);
	dc->FillRect(rcDraw, &brush);

	// Create the font to use to draw the tab text.
	CFont font;
	BOOL bSuccess = font.CreatePointFont(90, bBoldFont ? "Arial Bold" : "Arial", dc);
	_ASSERTE(bSuccess);
	CFont * pOriginalFont = dc->SelectObject(&font);

	// Draw the text with the same background color as the background.
	dc->SetBkMode(TRANSPARENT);
	strTabText.Format(" Tab #%d ", iCurrentTab);
	dc->DrawTextEx(strTabText, &lpDrawItemStruct->rcItem, DT_SINGLELINE | DT_VCENTER | DT_RIGHT, NULL);

	// Restore the original font.
	dc->SelectObject(pOriginalFont);
}

void CCustomTabCtrl::OnPaint()
{
	CString strTabText;
	CPaintDC dc(this); // device context for painting
	CString strTrace;
	
	// Find the tab currently selected and the number of tabs to draw.
	int iSelectedTab = GetCurSel();
	int iNumTabs = GetItemCount();

	// Draw all tabs.
	for(int iTabIndex=0; iTabIndex<iNumTabs; iTabIndex++)
	{
		// Get the bounding rectangle for this tab.
		CRect rcTab;
		BOOL bSuccess = GetItemRect(iTabIndex, &rcTab);

		// Draw the tab background.
		BOOL bSelectedTab = (iTabIndex == iSelectedTab);
		CBrush brush(bSelectedTab ? RGB(0xD4, 0xD0, 0xC8) : RGB(0xF7, 0xF3, 0xE9));
		dc.FillRect(rcTab, &brush);

		// Create the font to use to draw the tab text.
		CFont font;
		BOOL bBoldFont = (iSelectedTab == iTabIndex);
		bSuccess = font.CreatePointFont(90, bBoldFont ? "Arial Bold" : "Arial", &dc);
		_ASSERTE(bSuccess);
		CFont * pOriginalFont = dc.SelectObject(&font);

		// Get the text that is assigned to the current tab.
		TCITEM TabCtrlItem = {TCIF_TEXT};
		CString strTabText;
		TabCtrlItem.pszText = strTabText.GetBuffer(1024);
		TabCtrlItem.cchTextMax = 1024;
		bSuccess = GetItem(iTabIndex, &TabCtrlItem);
		strTabText.ReleaseBuffer();
		_ASSERTE(bSuccess);

		// Add two spaces after the tab string to give some space 
		// to the end of the tab (the strings are right-aligned).
		strTabText.Append("  ");

		// Draw the tab text with the same background color as the background.
		dc.SetBkMode(TRANSPARENT);
		dc.DrawTextEx(strTabText, rcTab, DT_SINGLELINE | DT_VCENTER | DT_RIGHT, NULL);

		// Restore the original font.
		dc.SelectObject(pOriginalFont);

		// If the tab is NOT selected we draw a gray line to the left of the tab.
		CPen grayPen(PS_SOLID, 1, RGB(0x80, 0x80, 0x80));
		CPen * pOldPen = dc.SelectObject(&grayPen);
		if(!bSelectedTab && iTabIndex > 0 && iTabIndex != iSelectedTab + 1)
		{
			strTrace.Format("Drawing gray line to the left (%d)\r\n", iTabIndex);
			OutputDebugString(strTrace);

			int x = rcTab.TopLeft().x - 1;
			int y1 = rcTab.TopLeft().y + 2;
			int y2 = rcTab.BottomRight().y - 2;
			dc.MoveTo(x, y1);
			dc.LineTo(x, y2);
		}

		// If the rightmost tab is NOT selected we draw a gray line to the right of the tab.
		if(!bSelectedTab && iTabIndex == iNumTabs - 1)
		{
			strTrace.Format("Drawing gray line to the right (%d)\r\n", iTabIndex);
			OutputDebugString(strTrace);

			int x = rcTab.TopLeft().x + rcTab.Width() - 1;
			int y1 = rcTab.TopLeft().y + 2;
			int y2 = rcTab.BottomRight().y - 2;
			dc.MoveTo(x, y1);
			dc.LineTo(x, y2);
		}

		// Draw a black line to the right of the tab if this is the selected tab.
		CPen blackPen(PS_SOLID, 1, RGB(0, 0, 0));
		dc.SelectObject(&blackPen);
		if(bSelectedTab)
		{
			strTrace.Format("Drawing black line to the right (%d)\r\n", iTabIndex);
			OutputDebugString(strTrace);

			int x = rcTab.TopLeft().x + rcTab.Width() - 1;
			int y1 = rcTab.TopLeft().y;
			int y2 = rcTab.BottomRight().y;
			dc.MoveTo(x, y1);
			dc.LineTo(x, y2);
		}

		// Restore the original pen.
		dc.SelectObject(&pOldPen);
	}
	
}

void CCustomTabCtrl::SetTabSize(int cx, int cy)
{
	SetItemSize(CSize(cx, cy));
	SetMinTabWidth(cx);
}

CAbstractTabPane * CCustomTabCtrl::GetTabPane(int iTabIndex)
{
	// Get the CAbstractTabPane-inherited object associated with the specified tab index.
	TCITEM tcItem = {TCIF_PARAM};
	BOOL bSuccess = GetItem(iTabIndex, &tcItem);
	_ASSERTE(bSuccess);
	CAbstractTabPane * pAbstractTabPane = (CAbstractTabPane *) tcItem.lParam;	

	// All ok.
	return pAbstractTabPane;
}

CAbstractTabPane * CCustomTabCtrl::GetCurrentTabPane()
{
	// Get the currently selected tab index.
	int iTabIndex = GetCurSel();

	// Get the CAbstractTabPane-inherited object associated with this tab.
	CAbstractTabPane * pAbstractTabPane = this->GetTabPane(iTabIndex);

	// All ok.
	return pAbstractTabPane;
}

