/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#include "stdafx.h"
#include "PciExplorer.h"
#include "PciBusView.h"
#include "MainFrm.h"
#include "mainfrm.h"
#include "PciFunctionView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_MEASUREITEM()
	ON_WM_MENUCHAR()
	ON_WM_INITMENUPOPUP()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
};

// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	// TODO: add member initialization code here
}

CMainFrame::~CMainFrame()
{
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}
	
	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | TBSTYLE_FLAT);

	VERIFY(m_wndToolBar.SetButtonText(0,"Refresh"));
	VERIFY(m_wndToolBar.SetButtonText(1,"About"));

	m_wndToolBar.SetSizes(CSize(45, 40), CSize(16,15));

	return 0;
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT /*lpcs*/,
	CCreateContext* pContext)
{
	BOOL bSuccess = m_wndSplitter.CreateStatic( this, 2, 1);
	_ASSERTE(bSuccess);

	bSuccess = m_wndSplitter.CreateView(0, 0, RUNTIME_CLASS(CPciBusView), CSize(0,0), pContext);
	_ASSERTE(bSuccess);

	bSuccess = m_wndSplitter.CreateView(1, 0, RUNTIME_CLASS(CPciFunctionView), CSize(0,0), pContext);
	_ASSERTE(bSuccess);

	// Spread the two panes evently over the client area.
	CRect rcClient;
	GetClientRect(&rcClient);
	HWND hWnd= m_wndSplitter.GetSafeHwnd();
	if(hWnd && ::IsWindow(hWnd))
	{
		int cy = rcClient.Height() / 2;
		m_wndSplitter.SetRowInfo(0, cy, 0);
		m_wndSplitter.SetRowInfo(1, cy, 0);
	}

	return (bSuccess);
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	// Remove the 'Untitled' title.
	cs.style &= ~FWS_ADDTOTITLE; 

	return TRUE;
}


// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG


HMENU CMainFrame::NewMenu()
{
	// Load the menu from the resources
	// ****replace IDR_MAINFRAME with your menu ID****
	m_menu.LoadMenu(IDR_MAINFRAME);

	// One method for adding bitmaps to menu options is 
	// through the LoadToolbar member function.This method 
	// allows you to add all the bitmaps in a toolbar object 
	// to menu options (if they exist). The function parameter 
	// is an the toolbar id. There is also a function called 
	// LoadToolbars that takes an array of id's.
	m_menu.LoadToolbar(IDR_MAINFRAME);

  return(m_menu.Detach());
}


void CMainFrame::OnMeasureItem(int nIDCtl, LPMEASUREITEMSTRUCT lpMeasureItemStruct)
{
	BOOL setflag=FALSE;
  if(lpMeasureItemStruct->CtlType==ODT_MENU){
    if(IsMenu((HMENU)lpMeasureItemStruct->itemID)){
      CMenu* cmenu =
       CMenu::FromHandle((HMENU)lpMeasureItemStruct->itemID);

      if(m_menu.IsMenu(cmenu)){
        m_menu.MeasureItem(lpMeasureItemStruct);
        setflag=TRUE;
      }
    }
  }

  if(!setflag)CFrameWnd::OnMeasureItem(nIDCtl, lpMeasureItemStruct);

}

LRESULT CMainFrame::OnMenuChar(UINT nChar, UINT nFlags, CMenu* pMenu)
{
LRESULT lresult;
  if(m_menu.IsMenu(pMenu))
    lresult=BCMenu::FindKeyboardShortcut(nChar, nFlags, pMenu);
  else
    lresult=CFrameWnd::OnMenuChar(nChar, nFlags, pMenu);
  return(lresult);
}

void CMainFrame::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu)
{
CFrameWnd::OnInitMenuPopup(pPopupMenu, nIndex, bSysMenu);
  if(!bSysMenu){
    if(m_menu.IsMenu(pPopupMenu))
      BCMenu::UpdateMenu(pPopupMenu);
  }
}
