/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#pragma once

//////////////////////////////////////////////
//
//	New Capabilities List structures.
//
//////////////////////////////////////////////

typedef enum CAPABILITY_ID { 

	CID_RESERVED,
	CID_PCI_POWER_MANAGEMENT,
	CID_AGP,
	CID_VPD,
	CID_SLOT_IDENTIFICATION,
	CID_MSI,
	CID_CPCI_HOT_SWAP,
	CID_PCIX,
	CID_AMD,
	CID_VENDOR_SPECIFIC,
	CID_DEBUG_PORT,
	CID_CPCI_CENTRAL_RESOURCE_CONTROL,
	CID_PCI_HOT_PLUG
};

typedef struct _CAPABILITIES_HEADER
{
	UCHAR ucCapabilityID;
	CHAR chNextCapabilityOffset;

}CAPABILITIES_HEADER, *PCAPABILITIES_HEADER;

typedef struct _CAPABILITY_PCI_POWER_MANAGEMENT
{
	CAPABILITIES_HEADER Hdr;
	WORD wPowerManagementCapabilities;
	WORD wControlStatusRegister;
	UCHAR ucBridgeSupportExtensions;
	UCHAR ucDataRegister;

}CAPABILITY_PCI_POWER_MANAGEMENT, *PCAPABILITY_PCI_POWER_MANAGEMENT;

typedef struct _CAPABILITY_AGP
{
	CAPABILITIES_HEADER Hdr;
	DWORD dwAgpStatusRegister;
	DWORD dwAgpCommandRegister;

}CAPABILITY_AGP, *PCAPABILITY_AGP;

typedef struct _CAPABILITY_VPD
{
	CAPABILITIES_HEADER Hdr;
	WORD wVpdAddressRegister;
	DWORD dwVpdDataRegister;

}CAPABILITY_VPD, *PCAPABILITY_VPD;

typedef struct _CAPABILITY_SLOT_IDENTIFICATION
{
	CAPABILITIES_HEADER Hdr;
	UCHAR ucExpansionSlotRegister;
	UCHAR ucChassisNumber;

}CAPABILITY_SLOT_IDENTIFICATION, *PCAPABILITY_SLOT_IDENTIFICATION;

typedef struct _CAPABILITY_MSI_32
{
	CAPABILITIES_HEADER Hdr;
	WORD wMessageControlRegister;
	DWORD dwMessageAddressRegister;
	WORD wMessageDataRegister;

}CAPABILITY_MSI_32, *PCAPABILITY_MSI_32;

typedef struct _CAPABILITY_MSI_64
{
	CAPABILITIES_HEADER Hdr;
	WORD wMessageControlRegister;
	DWORD dwMessageAddressRegisterLo;
	DWORD dwMessageAddressRegisterHi;
	WORD wMessageDataRegister;

}CAPABILITY_MSI_64, *PCAPABILITY_MSI_64;

typedef struct _CAPABILITY_MSI
{
	union
	{
		CAPABILITY_MSI_32 MsiCapsRegs32;
		CAPABILITY_MSI_64 MsiCapsRegs64;
	}u;

}CAPABILITY_MSI, *PCAPABILITY_MSI;

typedef struct _CAPABILITY_CPCI_HOT_SWAP
{
	CAPABILITIES_HEADER CapsHdr;

}CAPABILITY_CPCI_HOT_SWAP, *PCAPABILITY_CPCI_HOT_SWAP;

typedef struct _CAPABILITY_PCIX
{
	CAPABILITIES_HEADER CapsHdr;

}CAPABILITY_PCIX, *PCAPABILITY_PCIX;

typedef struct _CAPABILITY_AMD
{
	CAPABILITIES_HEADER CapsHdr;

}CAPABILITY_AMD, *PCAPABILITY_AMD;

typedef struct _CAPABILITY_VENDOR_SPECIFIC
{
	CAPABILITIES_HEADER CapsHdr;

}CAPABILITY_VENDOR_SPECIFIC, *PCAPABILITY_VENDOR_SPECIFIC;

typedef struct _CAPABILITY_DEBUG_PORT
{
	CAPABILITIES_HEADER CapsHdr;

}CAPABILITY_DEBUG_PORT, *PCAPABILITY_DEBUG_PORT;

typedef struct _CAPABILITY_CPCI_CENTRAL_RESOURCE_CONTROL
{
	CAPABILITIES_HEADER CapsHdr;

}CAPABILITY_CPCI_CENTRAL_RESOURCE_CONTROL, *PCPCI_CENTRAL_RESOURCE_CONTROL;

typedef struct _CAPABILITY_PCI_HOT_PLUG
{
	CAPABILITIES_HEADER CapsHdr;

}CAPABILITY_PCI_HOT_PLUG, *PCAPABILITY_PCI_HOT_PLUG;

//////////////////////////////////////////////
//
//	Utility structure to keep track of and 
//	sort the capability structures above.
//
//////////////////////////////////////////////

typedef struct _NEW_CAPS_ENTRY
{
	DWORD dwStartDwordOffset;
	PCAPABILITIES_HEADER pCapsHdr;

}NEW_CAPS_ENTRY, *PNEW_CAPS_ENTRY;
