/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#include "stdafx.h"
#include "PciExplorer.h"
#include "PciBusView.h"
#include "PciFunction.h"
#include "PciExplorerDoc.h"

IMPLEMENT_DYNCREATE(CPciBusView, CTreeView)

CPciBusView::CPciBusView()
{
}

CPciBusView::~CPciBusView()
{
}

BEGIN_MESSAGE_MAP(CPciBusView, CTreeView)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnTvnSelchanged)
	ON_COMMAND(IDM_REFRESH, OnRefresh)
END_MESSAGE_MAP()


// CPciBusView diagnostics

#ifdef _DEBUG
void CPciBusView::AssertValid() const
{
	CTreeView::AssertValid();
}

void CPciBusView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}
#endif //_DEBUG


// CPciBusView message handlers

int CPciBusView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CTreeView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// Get the tree control for later use.
	m_pTreeCtrl = &GetTreeCtrl();
	_ASSERTE(m_pTreeCtrl);

	// Set our background color to the nice yellow 
	// shade Visual Studio .NET uses for its tabs.
	m_pTreeCtrl->SetBkColor(RGB(0xF7, 0xF3, 0xE9));

	// Reconfigure the default tree control style.
	long lStyle = ::GetWindowLong(m_pTreeCtrl->m_hWnd, GWL_STYLE); 
	lStyle |= (TVS_HASLINES | TVS_LINESATROOT | TVS_HASBUTTONS | TVS_SHOWSELALWAYS | TVS_DISABLEDRAGDROP ); 
	::SetWindowLong(m_pTreeCtrl->m_hWnd, GWL_STYLE, lStyle); 

	// Set the Tree Control's imagelist.
	BOOL bSuccess = m_imageList.Create(IDB_TREEITEMS2, 16, 20, RGB(0, 255, 255));
	_ASSERTE(bSuccess);
	m_pTreeCtrl->SetImageList(&m_imageList, TVSIL_NORMAL);

	return 0;
}

void CPciBusView::OnInitialUpdate()
{
	CTreeView::OnInitialUpdate();
}

void CPciBusView::OnTvnSelchanged(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);

	// Make the lower pane update with the information of the selected node.
	GetDocument()->UpdateAllViews(this, UPDATE_SPECIFIED_PCI_FUNCTION, (CObject*) pNMTreeView->itemNew.lParam);
	
	// All ok.
	*pResult = 0;
}

void  CPciBusView::m_AddNodesToTreeControl(HTREEITEM hParent, CPciFunction * pPciFunction)
{
	// Build the VendorID/DeviceID string. We'll use it as the label in the tree control.
	CString strVendorDeviceIDs;
	strVendorDeviceIDs.Format("%d.%d.%d   %s   (%x/%x) - %s / %s", 
		pPciFunction->m_ulBus, pPciFunction->m_ulDevice, pPciFunction->m_ulFunction,
		pPciFunction->IsBridge() ? "Bridge" : "Function",
		pPciFunction->GetPciHeader()->Type0Hdr.usVendorID,
		pPciFunction->GetPciHeader()->Type0Hdr.usDeviceID,
		pPciFunction->GetVendorIdString().c_str(),
		pPciFunction->GetDeviceIdString().c_str());

	// Add the function to the tree control.
	TVINSERTSTRUCT tvis;
	tvis.hParent = hParent;
	tvis.hInsertAfter = hParent;
	tvis.item.mask = TVIF_HANDLE | TVIF_IMAGE | TVIF_PARAM | TVIF_SELECTEDIMAGE | TVIF_TEXT;
	tvis.item.hItem = NULL;
	tvis.item.iImage = m_GetImageIndex(pPciFunction);
	tvis.item.iSelectedImage = m_GetImageIndex(pPciFunction);
	tvis.item.lParam = (LPARAM) pPciFunction;
	tvis.item.pszText = (LPSTR)(const char*) strVendorDeviceIDs;
	HTREEITEM hThis = m_pTreeCtrl->InsertItem(&tvis);

	HTREEITEM hMultiFunctionDevice = NULL;
	ULONG ulMultiFunctionDeviceNumber = 0;

	// Add all child functions (if any) to the tree control.
	vector<CPciFunction*> * pvectSecondaryBusPciFunctions = pPciFunction->GetSecondaryBusPciFunctions();
	for(DWORD dwIndex = 0; dwIndex<pvectSecondaryBusPciFunctions->size(); dwIndex++)
	{
		// Get the next CPciFunction child node.
		pPciFunction = (*pvectSecondaryBusPciFunctions)[dwIndex];

		// If this CPciFunction object is the first function in a multi-function device we create 
		// a special 'Device' node that all the other functions in the same device will be put under.
		if(pPciFunction->GetPciHeader()->Type0Hdr.ucHeaderType & (1<<7) && pPciFunction->m_ulFunction == 0)
		{
			// Build the "Bus.Device.x Multi Function Device" string.
			strVendorDeviceIDs.Format("%d.%d.x   Multi Function Device", 
				pPciFunction->m_ulBus, pPciFunction->m_ulDevice);

			// Add the special 'Device' node.
			tvis.hParent = hThis;
			tvis.hInsertAfter = hThis;
			tvis.item.mask = TVIF_HANDLE | TVIF_IMAGE | TVIF_PARAM | TVIF_SELECTEDIMAGE | TVIF_TEXT;
			tvis.item.hItem = NULL;
			tvis.item.iImage = 2;
			tvis.item.iSelectedImage = 2;
			tvis.item.lParam = NULL;
			tvis.item.pszText = (LPTSTR) strVendorDeviceIDs.GetBuffer();
			hMultiFunctionDevice = m_pTreeCtrl->InsertItem(&tvis);

			// Get the device number from the first function in the multifunction device.
			// As long as we find PCI functions that belong to the same device we add them
			// under the 'Device' node. Note that all PCI functions in the vector reside 
			// on the same PCI bus so it is enough to check the device numbers.
			ulMultiFunctionDeviceNumber = pPciFunction->m_ulDevice;
		}

		// Recursively add child nodes to the tree control.
		HTREEITEM hParent = (hMultiFunctionDevice && pPciFunction->m_ulDevice == ulMultiFunctionDeviceNumber) ? hMultiFunctionDevice : hThis;
		m_AddNodesToTreeControl(hParent, pPciFunction);
	}
	
}

void CPciBusView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
	// pHint is really a pointer to the CPciFunction object that represents the Host/PCI bridge.
	CPciFunction * pHostPciBridge = (CPciFunction *) pHint;

	// If lHist is UPDATE_ALL_PCI_FUNCTIONS we should completely 
	// update the view with the information passed in pHint.
	if(lHint == UPDATE_ALL_PCI_FUNCTIONS && pHostPciBridge != NULL)
	{
		// Empty the previous view content.
		BOOL bSuccess = m_pTreeCtrl->DeleteAllItems();
		_ASSERTE(bSuccess);

		// Add all PCI functions to the tree control.
		m_AddNodesToTreeControl(TVI_ROOT, pHostPciBridge);

		// Select the root node.
		bSuccess = m_pTreeCtrl->SelectItem(m_pTreeCtrl->GetRootItem());	
		_ASSERTE(bSuccess);

		// Expand all nodes by default.
		m_pTreeCtrl->SendMessage(WM_KEYDOWN, (WPARAM)VK_MULTIPLY, (LPARAM)0); 
	}
}

int CPciBusView::m_GetImageIndex(CPciFunction * pPciFunction)
{
	static vClassCodeToImageMap[255] = {
		0,		// 00h: Unknown.
		10,		// 01h: Mass storage controller.
		4,		// 02h: Network Controller.
		5,		// 03h: Display Controller.
		17,		// 04h: Multimedia device.
		16,		// 05h: Memory Controller.
		1,		// 06h: Bridge Device.
		9,		// 07h: Simple communications controller.
		3,		// 08h: Base system peripherals.
		14,		// 09h: Input devices.
		11,		// 0Ah: Docking stations.
		16,		// 0Bh: Processors.
		18,		// 0Ch: Serial bus controllers.
	};

	// Find the image to use for the node in the treeview.
	PTYPEX_HEADER pTypeXHeader = pPciFunction->GetPciHeader();
	int iImage = vClassCodeToImageMap[pTypeXHeader->Type0Hdr.ucClassCode];

	// Differentiate between Floppy and other mass storage controllers.
	if(	pTypeXHeader->Type0Hdr.ucClassCode == 1 && 
		pTypeXHeader->Type0Hdr.ucSubClassCode == 2)
	{
		// Floppy.
		iImage = 11;
	}

	// Differentiate between USB, FireWire and other serial devices.
	if(pTypeXHeader->Type0Hdr.ucClassCode == 0xc)
	{
		switch(pTypeXHeader->Type0Hdr.ucSubClassCode)
		{
			case 0:
				// Firewire.
				iImage = 12;
				break;
			case 3:
				// USB.
				iImage = 18;
				break;
			default:
				// Unknown.
				iImage = 0;
		}
	}

	// Differentiate between Video, Audio and other multimedia devices.
	if(pTypeXHeader->Type0Hdr.ucClassCode == 0x4)
	{
		switch(pTypeXHeader->Type0Hdr.ucSubClassCode)
		{
			case 0:
				// Video.
				iImage = 19;
				break;
			case 1:
				// Audio.
				iImage = 17;
				break;
			default:
				// Unknown.
				iImage = 0;
		}
	}

	return iImage;
}
void CPciBusView::OnRefresh()
{
	CWaitCursor wait;

	CPciExplorerDoc * pPciExplorerDoc = (CPciExplorerDoc *) GetDocument();
	pPciExplorerDoc->ScanPciBus();
}
