/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#include "stdafx.h"
#include "PciExplorer.h"
#include "PciFunctionView.h"
#include "PciFunction.h"
#include "bcmenu.h"
#include "PciFunctionView.h"

IMPLEMENT_DYNCREATE(CPciFunctionView, CFormView)

CPciFunctionView::CPciFunctionView()
	: CFormView(CPciFunctionView::IDD)
{
	m_bHexadecimal = TRUE;
	m_bBarReadsOk = FALSE;
}

CPciFunctionView::~CPciFunctionView()
{

}

void CPciFunctionView::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CPciFunctionView, CFormView)
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(IDM_HEXADECIMAL, OnHexadecimal)
	ON_COMMAND(IDM_8BIT_DISPLAY, On8bitDisplay)
	ON_COMMAND(IDM_16BIT_DISPLAY, On16bitDisplay)
	ON_COMMAND(IDM_32BIT_DISPLAY, On32bitDisplay)
	ON_COMMAND(IDM__REFRESH, OnRefresh)
END_MESSAGE_MAP()


// CPciFunctionView diagnostics

#ifdef _DEBUG
void CPciFunctionView::AssertValid() const
{
	CFormView::AssertValid();
}

void CPciFunctionView::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}
#endif //_DEBUG

BOOL CPciFunctionView::m_InitializeTabControl()
{
	// Subclass the tab control defined in the dialog resources. 
	// The CCustomTabCtrl implements ownerdrawn tabs to show a 
	// more modern tab look.
	BOOL bSuccess = m_customTabCtrl.SubclassDlgItem(IDC_TAB, this);
	_ASSERTE(bSuccess);

	//////////////////////////////////////////////////
	// Add the default 'Configuration Space' tab.
	//////////////////////////////////////////////////

	// This tab exists for all devices. Additional tabs will be added
	// when a PCI Function is selected in the top PCI Bus View pane.
	m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM,  0, "Configuration Space", 0, (LPARAM) new CConfigSpacePane(this));

	// Adjust the sizes of the tabs.
	m_customTabCtrl.SetTabSize(125, 18);

	return TRUE;
}



void CPciFunctionView::OnInitialUpdate()
{
	// Make one-time initialization of various controls in the view.
	m_InitializeTabControl();

}

BOOL CPciFunctionView::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	///////////////////////////////////
	// Handle tab click notifications.
	///////////////////////////////////
	LPNMHDR pNmHdr = (LPNMHDR) lParam;
	if(pNmHdr->idFrom == IDC_TAB)
	{
		static int iOldSelTab;

		if(pNmHdr->code == TCN_SELCHANGING)
		{
			// Register the previously clicked tab.
			iOldSelTab = m_customTabCtrl.GetCurSel();
		}
		else if(pNmHdr->code == TCN_SELCHANGE)
		{
			// Register the currently clicked tab.
			int iNewSelTab = m_customTabCtrl.GetCurSel();

			// Switch from the previously selected 
			// tab to the currently selected tab.
			m_SwitchTab(iNewSelTab, iOldSelTab);
		}

		return TRUE;
	}

	///////////////////////////////////////
	// Handle edit control notifications.
	///////////////////////////////////////
	if(pNmHdr->idFrom == IDC_EDIT_VALUE)
	{
		// Get the row, column and new data.
		LV_DISPINFO * pLvDispInfo = (LV_DISPINFO *) pNmHdr;
		int iItemEdited = pLvDispInfo->item.iItem;
		int iSubItemEdited = pLvDispInfo->item.iSubItem;
		char * pszNewValue = pLvDispInfo->item.pszText;

		// Pass on the edit update to the active pane.
		CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetCurrentTabPane();
		pAbstractTabPane->OnValueEdited(iItemEdited, iSubItemEdited, pszNewValue);

		return TRUE;
	}

	/////////////////////////////////////////////
	// Handle Virtual list control notifications
	/////////////////////////////////////////////

	if(pNmHdr->code == LVN_GETDISPINFO)
	{ 
		// Pass on the notification to the active BAR pane.
		LV_DISPINFO * pDispInfo = (LV_DISPINFO *) lParam;
		CBarPane * pBarPane = (CBarPane *) m_customTabCtrl.GetCurrentTabPane();
		pBarPane->OnGetDispInfo(pDispInfo);

		return TRUE;
	}

	return TRUE;
}

BOOL CPciFunctionView::OnEraseBkgnd(CDC* pDC)
{
	// Set our background color to the nice yellow 
	// shade Visual Studio .NET uses for its tabs.
	CRect rcViewArea; 
    GetClientRect(&rcViewArea); 
    pDC->FillSolidRect(&rcViewArea, RGB(0xF7, 0xF3, 0xE9)); 

	// TRUE: Keeps GDI from painting.
	return TRUE;
}

void CPciFunctionView::OnSize(UINT nType, int cx, int cy)
{
	// Remove the scrollbars from the view.
	SetScrollSizes(MM_TEXT, CSize(cx, cy)); 
	CFormView::OnSize(nType, cx, cy);

	////////////////////////////////////////////////////////////////////
	// Resize the tab control width so that it covers the whole view.
	////////////////////////////////////////////////////////////////////

	// Has the tab control been created yet?
	CTabCtrl * pTabCtrl = (CTabCtrl *) GetDlgItem(IDC_TAB);
	if(!pTabCtrl)
		return;

	// Get the current position/size of the tab control. We need to do this
	// in order to preserve the height defined in the resource editor.
	CRect rcTabCtrl;
	pTabCtrl->GetWindowRect(&rcTabCtrl);

	// Set the new width of the tab control. Note that we don't move the window origin.
	pTabCtrl->SetWindowPos(this, 0 , 0, cx, rcTabCtrl.Height(), SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER);

	///////////////////////////////////////////////////////////////////////
	//	Resize the width of the panes so they cover the width of the view.
	///////////////////////////////////////////////////////////////////////

	// We are getting an WM_SIZE before we have subclassed 
	// the tab control so we check explicitly for this.
	if(m_customTabCtrl.m_hWnd)
	{
		int iNumTabs = m_customTabCtrl.GetItemCount();
		for(int iTabIndex=0; iTabIndex<iNumTabs; iTabIndex++)
		{
			// Resize the CAbstractTabPane-inherited object associated with this tab.
			CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetTabPane(iTabIndex);
			_ASSERTE(pAbstractTabPane);
			pAbstractTabPane->OnSize(nType, cx, cy);
		}
	}

	// Save the width and height of our window so that 
	// we can resize the panes when they are activated.
	m_icx = cx;
	m_icy = cy;
}

void CPciFunctionView::m_SwitchTab(int iNewSelTab, int iOldSelTab)
{
	// If the user clicked an already selected tab we don't need to do anything.
	if(iNewSelTab == iOldSelTab)
	{
		return;
	}

	// Get the CAbstractTabPane-inherited object associated with the clicked tab.
	CAbstractTabPane * pClickedTabPane = m_customTabCtrl.GetTabPane(iNewSelTab);

	// If the user selected one of the 'BAR' tabs we need
	// to get permission before switching to the tab.
	if(pClickedTabPane->GetType() == BarPane)
	{
		if(!m_bBarReadsOk)
		{
			// Temporarily set the active tab to the old tab while we display the message box.
			// This is to give the impression that the message box was shown BEFORE the tab change.
			m_customTabCtrl.SetCurSel(iOldSelTab);

			if(MessageBox("Modifying the Memory or I/O range associated with a BAR might\n"
				"have device-dependent side-effects. Do you want to continue?", "Warning", MB_ICONWARNING | MB_YESNO) == IDNO)
			{
				// The user did not want to read device memory or I/O so we need to switch back to the previous pane. 
				m_SwitchTab(iOldSelTab, iNewSelTab);
				
				// We're done.
				return;
			}	

			// The user has ok'd showing the BAR tab.
			m_bBarReadsOk = TRUE;
		}
	}

	// In case this function was called to switch the tab (i.e. the tab has 
	// not yet been clicked) we make sure that the new tab is selected.
	m_customTabCtrl.SetCurSel(iNewSelTab);

	// Resize the pane we are about to activate to make 
	// sure it fills the area under the tab control.
	pClickedTabPane->OnSize(0, m_icx, m_icy);

	// Activate the new tab pane.
	pClickedTabPane->ActivatePane();

	// Deactivate all tabs except the clicked tab.
	int iNumTabs = m_customTabCtrl.GetItemCount();
	for(int iTabIndex=0; iTabIndex<iNumTabs; iTabIndex++)
	{
		// Get the CAbstractTabPane-inherited object associated with this tab.
		CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetTabPane(iTabIndex);
		_ASSERTE(pAbstractTabPane);
		
		// Deactivate all tabs except the clicked tab.
		if(pAbstractTabPane != pClickedTabPane)
		{
			pAbstractTabPane->DeactivatePane();	
		} 
	}

	// Show the content of the selected pane.
	pClickedTabPane->ShowPciFunction(m_pCachedPciFunction);
}

void CPciFunctionView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
	// Update the view with the information contained in the CPciFunction object passed in.
	if(lHint == UPDATE_SPECIFIED_PCI_FUNCTION)
	{
		// pHint is really a pointer to the CPciFunction object selected in the upper pane.
		m_pCachedPciFunction = (CPciFunction *) pHint;

		// Switch to the "Configuration Space" tab which is the default tab.
		m_SwitchTab(0, -1);

		// Enable the tabs that correspond to the features in the PCI Function.
		m_EnableTabsForPciFunctionFeatures(m_pCachedPciFunction);

		// Update the 'Configuration Space' pane.
		CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetTabPane(0);
		_ASSERTE(pAbstractTabPane);
		pAbstractTabPane->ShowPciFunction(m_pCachedPciFunction);
	}
}

void CPciFunctionView::OnContextMenu(CWnd* pWnd, CPoint point)
{
	// Ask the active pane to display its custom context menu.
	CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetCurrentTabPane();
	pAbstractTabPane->OnContextMenu(pWnd, point);
}

void CPciFunctionView::OnHexadecimal()
{
	// Toggle the hex/decimal value display.
	m_bHexadecimal = !m_bHexadecimal;

	// Tell all panes to redisplay the current PCI Function in Hexadecimal notation.
	int iNumTabs = m_customTabCtrl.GetItemCount();
	for(int iTabIndex=0; iTabIndex<iNumTabs; iTabIndex++)
	{
		// Get the CAbstractTabPane-inherited object associated with this tab.
		CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetTabPane(iTabIndex);
		_ASSERTE(pAbstractTabPane);

		// Deactivate all tabs except the clicked tab.
		pAbstractTabPane->SetHexadecimalDisplay(m_bHexadecimal);
	}
}

void CPciFunctionView::m_EnableTabsForPciFunctionFeatures(CPciFunction * pPciFunction)
{
	// Remove all tabs except the first which is the Configuration Space Tab
	// (all PCI Functions have at least the 'Configuration Space' tab).
	// NOTE: We delete the tabs from higher to lower indices or else the 
	// remaining tab indices will change as we delete tabs.
	int iNumTabs = m_customTabCtrl.GetItemCount();
	for(int iTabIndex=iNumTabs-1; iTabIndex>0; iTabIndex--)
	{
		// Get the CAbstractTabPane-inherited object associated with this tab.
		CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetTabPane(iTabIndex);
		
		// Delete this pane object.
		delete pAbstractTabPane;

		// Delete this tab from the tab control.
		m_customTabCtrl.DeleteItem(iTabIndex);
	}

	// If this is a multifunction device we don't add any more tabs.
	if(!pPciFunction)
		return;

	// Enable BAR0..BAR5 tabs that are supported by this PCI Function.
	PTYPEX_HEADER pTypeXHdr = pPciFunction->GetPciHeader();
	BOOL b64BitMemBar = FALSE;

	switch(pTypeXHdr->Type0Hdr.ucHeaderType & 0x7f)
	{
		case 0:
			if(m_IsBarUsed(pTypeXHdr->Type0Hdr.ulBAR1, pTypeXHdr->Type0Hdr.ulBAR0, &b64BitMemBar))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 1, "BAR 0", 0, (LPARAM) new CBarPane(this, 0));
			}

			if(!b64BitMemBar && m_IsBarUsed(0, pTypeXHdr->Type0Hdr.ulBAR1, NULL))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 2, "BAR 1", 0, (LPARAM) new CBarPane(this, 1));
			}

			if(m_IsBarUsed(pTypeXHdr->Type0Hdr.ulBAR3, pTypeXHdr->Type0Hdr.ulBAR2, &b64BitMemBar))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 3, "BAR 2", 0, (LPARAM) new CBarPane(this, 2));
			}	
		
			if(!b64BitMemBar && m_IsBarUsed(0, pTypeXHdr->Type0Hdr.ulBAR3, NULL))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 4, "BAR 3", 0, (LPARAM) new CBarPane(this, 3));
			}	
		
			if(m_IsBarUsed(pTypeXHdr->Type0Hdr.ulBAR5, pTypeXHdr->Type0Hdr.ulBAR4, &b64BitMemBar))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 5, "BAR 4", 0, (LPARAM) new CBarPane(this, 4));
			}	

			if(!b64BitMemBar && m_IsBarUsed(0, pTypeXHdr->Type0Hdr.ulBAR5, NULL))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 6, "BAR 5", 0, (LPARAM) new CBarPane(this, 5));
			}	
			break;

		case 1:

			if(m_IsBarUsed(pTypeXHdr->Type1Hdr.ulBAR1, pTypeXHdr->Type1Hdr.ulBAR0, &b64BitMemBar))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 1, "BAR 0", 0, (LPARAM) new CBarPane(this, 0));
			}

			if(!b64BitMemBar && m_IsBarUsed(0, pTypeXHdr->Type1Hdr.ulBAR1, NULL))
			{
				m_customTabCtrl.InsertItem(TCIF_TEXT | TCIF_PARAM, 2, "BAR 1", 0, (LPARAM) new CBarPane(this, 1));
			}
	}
}

BOOL CPciFunctionView::m_IsBarUsed(DWORD dwBarHigh, DWORD dwBarLow, BOOL * pb64BitMemBar)
{
	BOOL bBarUsed = FALSE;

	// We assume that a zero BAR means it is not implemented.
	if(dwBarLow == 0)
	{
		return FALSE;
	}

	BOOL bMemoryBar = (dwBarLow & (1<<0)) == 0;
	if(bMemoryBar)
	{
		// Memory BAR.
		UCHAR ucType = (UCHAR) ((dwBarLow>>1) & 3);
		BOOL b64BitMemBar = (ucType == 2);
		BOOL bPrefetchable = dwBarLow & (1<<3);
		DWORD dwBaseAddress = dwBarLow>>4;

		// Set out-parameters.
		bBarUsed = dwBaseAddress != 0;
		if(pb64BitMemBar)
		{
			*pb64BitMemBar = b64BitMemBar;
		}
	}
	else
	{
		// I/O BAR.
		DWORD dwBaseAddress = dwBarLow>>2;
		
		// Set out-parameters.
		bBarUsed = dwBaseAddress != 0;
		if(pb64BitMemBar)
		{
			*pb64BitMemBar = FALSE;
		}
	}

	return bBarUsed;
}

void CPciFunctionView::On8bitDisplay()
{
	// This message came from the CBarPane class's context menu so we forward.
	CBarPane * pBarPane = (CBarPane *) m_customTabCtrl.GetCurrentTabPane();
	pBarPane->On8bitDisplay();
}

void CPciFunctionView::On16bitDisplay()
{
	// This message came from the CBarPane class's context menu so we forward.
	CBarPane * pBarPane = (CBarPane *) m_customTabCtrl.GetCurrentTabPane();
	pBarPane->On16bitDisplay();
}

void CPciFunctionView::On32bitDisplay()
{
	// This message came from the CBarPane class's context menu so we forward.
	CBarPane * pBarPane = (CBarPane *) m_customTabCtrl.GetCurrentTabPane();
	pBarPane->On32bitDisplay();
}

void CPciFunctionView::OnRefresh()
{
	// This message came from the active pane's 
	// context menu. Refresh the active Pane.
	CAbstractTabPane * pAbstractTabPane = m_customTabCtrl.GetCurrentTabPane();
	pAbstractTabPane->OnRefresh();
}
