/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#pragma once

#include "Resource.h"
#include "CustomTabCtrl.h"
#include "PciFunction.h"
#include "ConfigSpacePane.h"
#include "BarPane.h"

class CConfigSpacePane;
class CBarPane;

class CPciFunctionView : public CFormView
{
	DECLARE_DYNCREATE(CPciFunctionView)

protected:
	CPciFunctionView();           // protected constructor used by dynamic creation
	virtual ~CPciFunctionView();

	virtual void OnInitialUpdate( );

public:
	enum { IDD = IDD_PCIFUNCTIONVIEW };
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

private:

	//////////////////////////////
	//	Member variables.
	/////////////////////////////

	// This determines if the values in the panes 
	// are displayed as hexadecimal or decimal.
	BOOL m_bHexadecimal;

	// This is the custom tab control that we'll use to subclass 
	// the tab control defined in the dialog resources.
	CCustomTabCtrl m_customTabCtrl;

	// This is a cached pointer to the currently displayed CPciFunction object.
	CPciFunction * m_pCachedPciFunction;

	// This flag tells us if the user has ok'd to read and view the memory or I/O of the BARs.
	// We warn the user about reading BAR memory or I/O the first time a BAR tab is activated.
	BOOL m_bBarReadsOk;

	// These variables hold the width and height of our view.
	// They are updated when the window is resized. We use them
	// to resize the panes when tab panes are activated.
	int m_icx;
	int m_icy;

	//////////////////////////////
	//	Helper functions.
	/////////////////////////////

	BOOL m_InitializeTabControl();
	void m_EnableTabsForPciFunctionFeatures(CPciFunction * pPciFunction);
	void m_SwitchTab(int iNewSelTab, int iOldSelTab);
	BOOL m_IsBarUsed(DWORD dwBarHigh, DWORD dwBarLow, BOOL * pb64BitMemBar);

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()

protected:
	virtual BOOL OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult);
public:
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);

	afx_msg void OnSize(UINT nType, int cx, int cy);

protected:
	virtual void OnUpdate(CView* /*pSender*/, LPARAM /*lHint*/, CObject* /*pHint*/);
public:
	afx_msg void OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/);
	afx_msg void OnHexadecimal();

	afx_msg void On8bitDisplay();
	afx_msg void On16bitDisplay();
	afx_msg void On32bitDisplay();
	afx_msg void OnRefresh();
};


