/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#include "pch.h"
#include "..\intrface.h"
#ifdef PCIEXDRV_WMI_TRACE
#include "pciexdrv.tmh"
#endif

// global data
PCIEXDRV_DATA g_Data;

NTSTATUS CompleteIrp(PIRP pIrp, ULONG ulInfo, NTSTATUS status)
{
	pIrp->IoStatus.Information = ulInfo;
	pIrp->IoStatus.Status = status;
	IoCompleteRequest(pIrp, IO_NO_INCREMENT);

	return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  DriverEntry 
//      Installable driver initialization entry point.
//      This entry point is called directly by the I/O system.
//
//  Arguments:
//      IN  DriverObject
//              pointer to the driver object
//
//      IN  RegistryPath
//              pointer to a unicode string representing the path,
//              to driver-specific key in the registry.
//
//  Return Value:
//      Status
//
NTSTATUS DriverEntry(
    IN  PDRIVER_OBJECT  DriverObject,
    IN  PUNICODE_STRING RegistryPath
    )
{
    NTSTATUS                            status;
    PDEVICE_OBJECT                      deviceObject;
    PPCIEXDRV_DEVICE_EXTENSION   deviceExtension;
    UNICODE_STRING                      ntName;
    UNICODE_STRING                      win32Name;

    pciexdrvDebugPrint(DBG_INIT, DBG_TRACE, __FUNCTION__"++");
    pciexdrvDebugPrint(DBG_INIT, DBG_INFO, "Compiled at %s on %s", __TIME__, __DATE__);

#ifdef DBG
//    DbgBreakPoint();
#endif

#ifdef PCIEXDRV_WMI_TRACE 
    WPP_INIT_TRACING(DriverObject, RegistryPath);
#endif

    RtlZeroMemory(&g_Data, sizeof(PCIEXDRV_DATA));

    // save registry path
    g_Data.RegistryPath.Length = RegistryPath->Length;
    g_Data.RegistryPath.MaximumLength = RegistryPath->Length + sizeof(UNICODE_NULL);
    g_Data.RegistryPath.Buffer = (PWCHAR)ExAllocatePoolWithTag(
                                            PagedPool,
                                            g_Data.RegistryPath.MaximumLength,
                                            PCIEXDRV_POOL_TAG
                                            );

    if (g_Data.RegistryPath.Buffer == NULL)
    {
        status = STATUS_INSUFFICIENT_RESOURCES;

        pciexdrvDebugPrint(DBG_INIT, DBG_ERR, __FUNCTION__": Failed to allocate memory for RegistryPath");

        return status;
    }

    RtlCopyUnicodeString(&g_Data.RegistryPath, RegistryPath);

    // setup our dispatch function table in the driver object
    DriverObject->MajorFunction[IRP_MJ_CREATE] = pciexdrvCreateDispatch;
    DriverObject->MajorFunction[IRP_MJ_CLOSE] = pciexdrvCloseDispatch;
    DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = pciexdrvDeviceIoControlDispatch;
    DriverObject->MajorFunction[IRP_MJ_CLEANUP] = pciexdrvCleanupDispatch;
    DriverObject->MajorFunction[IRP_MJ_SHUTDOWN] = pciexdrvShutdownDispatch;
    DriverObject->DriverUnload = pciexdrvUnload;

    // initialize device name
    RtlInitUnicodeString(&ntName, L"\\Device\\pciexdrvDevice");

    // Create our function device object.
    status = IoCreateDevice(
                DriverObject,
                sizeof (PCIEXDRV_DEVICE_EXTENSION),
                &ntName,
                FILE_DEVICE_UNKNOWN,
                0,
                FALSE,
                &deviceObject
                );

    if (!NT_SUCCESS (status)) 
    {
        ExFreePool(g_Data.RegistryPath.Buffer);
        g_Data.RegistryPath.Buffer = NULL;

        pciexdrvDebugPrint(DBG_INIT, DBG_ERR, __FUNCTION__"--. STATUS %x", status);

        return status;
    }

    // Initialize the device extension.
    deviceExtension = (PPCIEXDRV_DEVICE_EXTENSION)deviceObject->DeviceExtension;

    // Zero the memory
    RtlZeroMemory(deviceExtension, sizeof(PCIEXDRV_DEVICE_EXTENSION));

    // save our device object pointer
    deviceExtension->DeviceObject = deviceObject;

    // This flag sets the buffering method for reads and writes
    // to METHOD_DIRECT.  IOCTLs are handled by IO control codes
    // independent of the value of this flag.
    deviceObject->Flags |= DO_DIRECT_IO;

    RtlInitUnicodeString(&win32Name, L"\\??\\pciexdrvDevice");
    status = IoCreateSymbolicLink(&win32Name, &ntName);
    if (!NT_SUCCESS(status))
    {
        IoDeleteDevice(deviceObject);

        ExFreePool(g_Data.RegistryPath.Buffer);
        g_Data.RegistryPath.Buffer = NULL;

        return status;
    }

    IoRegisterShutdownNotification(deviceObject);

    pciexdrvDebugPrint(DBG_INIT, DBG_TRACE, __FUNCTION__"--. STATUS %x", status);

    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  pciexdrvCreateDispatch
//      Dispatch routine for IRP_MJ_CREATE requests.
//
//  Arguments:
//      IN  DeviceObject
//              pointer to the device object for our device
//
//      IN  Irp
//              the create IRP
//
//  Return Value:
//      NT status code.
//
NTSTATUS pciexdrvCreateDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PPCIEXDRV_DEVICE_EXTENSION    deviceExtension;
    NTSTATUS                        status;

    pciexdrvDebugPrint(DBG_CREATECLOSE, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PPCIEXDRV_DEVICE_EXTENSION)DeviceObject->DeviceExtension;

    if (InterlockedIncrement(&deviceExtension->OpenHandleCount) > 1)
    {
/*        status = STATUS_SHARING_VIOLATION;

        InterlockedDecrement(&deviceExtension->OpenHandleCount);

        Irp->IoStatus.Information = 0;
        Irp->IoStatus.Status = status;
        IoCompleteRequest(Irp, IO_NO_INCREMENT);

        pciexdrvDebugPrint(DBG_CREATECLOSE, DBG_WARN, __FUNCTION__"--. IRP %p, STATUS %x", Irp, status);

        return status;
*/    }

    status = STATUS_SUCCESS;

    Irp->IoStatus.Information = 0;
    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);

    pciexdrvDebugPrint(DBG_CREATECLOSE, DBG_TRACE, __FUNCTION__"--. IRP %p, STATUS %x", Irp, status);

    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  pciexdrvCloseDispatch
//      Dispatch routine for IRP_MJ_CLOSE requests.
//
//  Arguments:
//      IN  DeviceObject
//              pointer to the device object for our device
//
//      IN  Irp
//              the close IRP
//
//  Return Value:
//      NT status code.
//
NTSTATUS pciexdrvCloseDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PPCIEXDRV_DEVICE_EXTENSION    deviceExtension;
    NTSTATUS                        status;

    pciexdrvDebugPrint(DBG_CREATECLOSE, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PPCIEXDRV_DEVICE_EXTENSION)DeviceObject->DeviceExtension;

    status = STATUS_SUCCESS;

    Irp->IoStatus.Information = 0;
    Irp->IoStatus.Status = status;
    IoCompleteRequest (Irp, IO_NO_INCREMENT);

    InterlockedDecrement(&deviceExtension->OpenHandleCount);

    pciexdrvDebugPrint(DBG_CREATECLOSE, DBG_TRACE, __FUNCTION__"--. IRP %p, STATUS %x", Irp, status);

    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  pciexdrvCleanupDispatch
//      Dispatch routine for IRP_MJ_CLEANUP requests.
//
//  Arguments:
//      IN  DeviceObject
//              pointer to the device object for our device
//
//      IN  Irp
//              the create IRP
//
//  Return Value:
//      NT status code.
//
NTSTATUS pciexdrvCleanupDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PPCIEXDRV_DEVICE_EXTENSION    deviceExtension;
    NTSTATUS                status;
    PIO_STACK_LOCATION      irpStack;

    pciexdrvDebugPrint(DBG_CREATECLOSE, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PPCIEXDRV_DEVICE_EXTENSION)DeviceObject->DeviceExtension;

    irpStack = IoGetCurrentIrpStackLocation(Irp);
    pciexdrvFlushQueues(deviceExtension, irpStack->FileObject);

    status = STATUS_SUCCESS;

    Irp->IoStatus.Information = 0;
    Irp->IoStatus.Status = status;
    IoCompleteRequest(Irp, IO_NO_INCREMENT);

    pciexdrvDebugPrint(DBG_CREATECLOSE, DBG_TRACE, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  pciexdrvUnload
//      Driver unload callback.
//
//  Arguments:
//      IN  DriverObject
//              pointer to the driver object
//
//  Return Value:
//      none
//
VOID pciexdrvUnload(
    IN  PDRIVER_OBJECT  DriverObject
    )
{
    UNICODE_STRING  win32Name;

    pciexdrvDebugPrint(DBG_UNLOAD, DBG_TRACE, __FUNCTION__"++");

    RtlInitUnicodeString(&win32Name, L"\\??\\pciexdrvDevice");
    IoDeleteSymbolicLink(&win32Name);

    IoUnregisterShutdownNotification(DriverObject->DeviceObject);

    IoDeleteDevice(DriverObject->DeviceObject);

    // The device object(s) should be NULL now
    // (since we unload, all the devices objects associated with this
    // driver must be deleted.
    ASSERT(DriverObject->DeviceObject == NULL);

    // We should not be unloaded until all the devices we control
    // have been removed from our queue.

    // release memory block allocated for registry path
    if (g_Data.RegistryPath.Buffer != NULL)
    {
        ExFreePool(g_Data.RegistryPath.Buffer);
        g_Data.RegistryPath.Buffer = NULL;
    }

    pciexdrvDebugPrint(DBG_UNLOAD, DBG_TRACE, __FUNCTION__"--");

#ifdef PCIEXDRV_WMI_TRACE
    WPP_CLEANUP(DriverObject);
#endif

    return;
}

ULONG ReadConfigDword(ULONG ulBus, ULONG ulDevice, ULONG ulFunction, ULONG ulDwordOffset)
{
	ULONG ulOldConfigAddress;
	ULONG ulConfigAddress;
	ULONG ulData;

	// Save the existing content of the configuration address port in the host/PCI bridge.
	ulOldConfigAddress = READ_PORT_ULONG((PULONG)0xcf8);

	// Setup the configuration address port in the host/PCI bridge.
	ulConfigAddress = (1<<31) | (ulBus << 16) | (ulDevice << 11) | (ulFunction << 8) | (ulDwordOffset << 2);
	WRITE_PORT_ULONG((PULONG)0xcf8, ulConfigAddress);

	// Read the addressed DWORD from the configuration data port of the host/PCI bridge.
	ulData = READ_PORT_ULONG((PULONG)0xcfc);

	// Restore the original configuration address port content.
	WRITE_PORT_ULONG((PULONG)0xcf8, ulOldConfigAddress);

	// All OK.
	return ulData;
}

void WriteConfigXxx(ULONG ulBus, ULONG ulDevice, ULONG ulFunction, ULONG ulByteOffset, UCHAR ucSize, ULONG ulData)
{
	ULONG ulOldConfigAddress;
	ULONG ulConfigAddress;
	ULONG ulWriteOffset;

	// Save the existing content of the configuration address port in the host/PCI bridge.
	ulOldConfigAddress = READ_PORT_ULONG((PULONG)0xcf8);

	// Setup the configuration address port in the host/PCI bridge.
	// NOTE: The offset passed in here is the BYTE offset within the header.
	// Because the configuration address port takes the index of the DWORD
	// we must shift the ulOffset to the right 2 bits (and thus dividing by 4).
	// Since we normally would shift the ulOffset up by 2 bits we now simply
	// don't shift at all. We must however mask out the lower two bits since
	// they are used below to specify the offset to write within the addressed 
	// DWORD.
	ulConfigAddress = (1<<31) | (ulBus << 16) | (ulDevice << 11) | (ulFunction << 8) | (ulByteOffset & ~3);
	WRITE_PORT_ULONG((PULONG)0xcf8, ulConfigAddress);

	// Write the addressed BYTE, WORD or DWORD to the configuration data port of the host/PCI bridge.
	// NOTE: The two least significant bits of ulOffset indicate where in the configuration data port
	// we write the data. The host/PCI bridge will pass through BYTE, WORD and DWORD writes as config
	// write transactions of the same size on the PCI bus.
	ulWriteOffset = 0xcfc + (ulByteOffset & 3);
	switch(ucSize)
	{
		case 1:
			WRITE_PORT_UCHAR((PUCHAR) ulWriteOffset, (UCHAR) ulData);
			break;
		case 2:
			WRITE_PORT_USHORT((PUSHORT) ulWriteOffset, (USHORT) ulData);
			break;
		case 4:
			WRITE_PORT_ULONG((PULONG) ulWriteOffset, (ULONG) ulData);
	}

	// Restore the original configuration address port content.
	WRITE_PORT_ULONG((PULONG)0xcf8, ulOldConfigAddress);
}

ULONG ReadConfigXxx(ULONG ulBus, ULONG ulDevice, ULONG ulFunction, ULONG ulByteOffset, UCHAR ucSize)
{
	ULONG ulOldConfigAddress;
	ULONG ulConfigAddress;
	ULONG ulReadOffset;
	ULONG ulData;

	// Save the existing content of the configuration address port in the host/PCI bridge.
	ulOldConfigAddress = READ_PORT_ULONG((PULONG)0xcf8);

	// Setup the configuration address port in the host/PCI bridge.
	// NOTE: The offset passed in here is the BYTE offset within the header.
	// Because the configuration address port takes the index of the DWORD
	// we must shift the ulOffset to the right 2 bits (and thus dividing by 4).
	// Since we normally would shift the ulOffset up by 2 bits we now simply
	// don't shift at all. We must however mask out the lower two bits since
	// they are used below to specify the offset to read within the addressed 
	// DWORD.
	ulConfigAddress = (1<<31) | (ulBus << 16) | (ulDevice << 11) | (ulFunction << 8) | (ulByteOffset & ~3);
	WRITE_PORT_ULONG((PULONG)0xcf8, ulConfigAddress);

	// Read the addressed BYTE, WORD or DWORD from the configuration data port of the host/PCI bridge.
	// NOTE: The two least significant bits of ulOffset indicate where in the configuration data port
	// we read the data. The host/PCI bridge will pass through BYTE, WORD and DWORD reads as config
	// read transactions of the same size on the PCI bus.
	ulReadOffset = 0xcfc + (ulByteOffset & 3);
	switch(ucSize)
	{
		case 1:
			ulData = READ_PORT_UCHAR((PUCHAR) ulReadOffset);
			break;
		case 2:
			ulData = READ_PORT_USHORT((PUSHORT) ulReadOffset);
			break;
		case 4:
			ulData = READ_PORT_ULONG((PULONG) ulReadOffset);
	}

	// Restore the original configuration address port content.
	WRITE_PORT_ULONG((PULONG)0xcf8, ulOldConfigAddress);

	// All ok.
	return ulData;
}

void ReadDeviceData(
	PREAD_DEVICE_DATA_INPARAMS pReadDeviceDataInParams,
	PREAD_DEVICE_DATA_OUTPARAMS pReadDeviceDataOutParams)
{
	UCHAR ucMemorySpace = pReadDeviceDataInParams->ucMemorySpace;			// 0: I/O Space, 1: Mem Space.
	ULONG ulPhysicalAddress = pReadDeviceDataInParams->ulPhysicalAddress;	// Dword-aligned offset to read from.
	ULONG ulItemSize = pReadDeviceDataInParams->ulItemSize;					// 1: Byte, 2: Word, 4: Dword.
	ULONG ulItemCount = pReadDeviceDataInParams->ulItemCount;				// Number of bytes, words or dwords.
	
	ULONG ulIndex;

	// Find out if we are reading memory or I/O space.
	if(ucMemorySpace)
	{
		// Map the physical address passed in to a virtual address we can read.
		PHYSICAL_ADDRESS paPhysicalAddress = {ulPhysicalAddress, 0};
		ULONG ulNumBytes = ulItemSize * ulItemCount;
		PVOID pVirtualAddress = MmMapIoSpace(paPhysicalAddress, ulNumBytes, MmNonCached);
		ASSERT(pVirtualAddress != NULL);

		// Read <ulItemCount> <ulItemSize>-sized items from address <ulOffset> in memory space.
		switch(ulItemSize)
		{
			case 1:
				// Read <ulItemCount> bytes.
				READ_REGISTER_BUFFER_UCHAR(
					pVirtualAddress, 
					pReadDeviceDataOutParams->data.vucDataBuf, 
					ulItemCount);
				break;

			case 2:
				// Read <ulItemCount> words.
				READ_REGISTER_BUFFER_USHORT(
					pVirtualAddress,
					pReadDeviceDataOutParams->data.vusDataBuf,
					ulItemCount);
				break;

			case 4:
				// Read <ulItemCount> dwords.
				READ_REGISTER_BUFFER_ULONG(
					pVirtualAddress,
					pReadDeviceDataOutParams->data.vulDataBuf,
					ulItemCount);
				break;
		}

		// Unmap the virtual address.
		MmUnmapIoSpace(pVirtualAddress, ulNumBytes);
	}
	else
	{
		PUCHAR pucPhysicalAddress = (PUCHAR) ulPhysicalAddress;
		PUSHORT pusPhysicalAddress = (PUSHORT) ulPhysicalAddress;
		PULONG pulPhysicalAddress = (PULONG) ulPhysicalAddress;

		// Read <ulItemCount> <ulItemSize>-sized items from address <ulOffset> in I/O space.
		switch(ulItemSize)
		{
			case 1:
				// Read <ulItemCount> bytes.
				for(ulIndex=0; ulIndex<ulItemCount; ulIndex++)
				{
					pReadDeviceDataOutParams->data.vucDataBuf[ulIndex] = READ_PORT_UCHAR(pucPhysicalAddress + ulIndex);
				}
				break;

			case 2:
				// Read <ulItemCount> words.
				for(ulIndex=0; ulIndex<ulItemCount; ulIndex++)
				{
					pReadDeviceDataOutParams->data.vusDataBuf[ulIndex] = READ_PORT_USHORT(pusPhysicalAddress + ulIndex);
				}
				break;

			case 4:
				// Read <ulItemCount> dwords.
				for(ulIndex=0; ulIndex<ulItemCount; ulIndex++)
				{
					pReadDeviceDataOutParams->data.vulDataBuf[ulIndex] = READ_PORT_ULONG(pulPhysicalAddress + ulIndex);
				}
				break;
		}
	}
}

void WriteDeviceData(PWRITE_DEVICE_DATA_INPARAMS pWriteDeviceDataInParams)
{
	UCHAR ucMemorySpace = pWriteDeviceDataInParams->ucMemorySpace;			// 0: I/O Space, 1: Mem Space.
	ULONG ulPhysicalAddress = pWriteDeviceDataInParams->ulPhysicalAddress;	// Dword-aligned offset to write to.
	ULONG ulItemSize = pWriteDeviceDataInParams->ulItemSize;				// 1: Byte, 2: Word, 4: Dword.
	ULONG ulItemCount = pWriteDeviceDataInParams->ulItemCount;				// Number of bytes, words or dwords.

	ULONG ulIndex;

	// Find out if we are writing memory or I/O space.
	if(ucMemorySpace)
	{
		// Map the physical address passed in to a virtual address we can write.
		PHYSICAL_ADDRESS paPhysicalAddress = {ulPhysicalAddress, 0};
		ULONG ulNumBytes = ulItemSize * ulItemCount;
		PVOID pVirtualAddress = MmMapIoSpace(paPhysicalAddress, ulNumBytes, MmNonCached);
		ASSERT(pVirtualAddress != NULL);

		// Write <ulItemCount> <ulItemSize>-sized items to address <ulOffset> in memory space.
		switch(ulItemSize)
		{
			case 1:
				// Write <ulItemCount> bytes.
				WRITE_REGISTER_BUFFER_UCHAR(
					pVirtualAddress, 
					pWriteDeviceDataInParams->data.vucDataBuf, 
					ulItemCount);
				break;

			case 2:
				// Write <ulItemCount> words.
				WRITE_REGISTER_BUFFER_USHORT(
					pVirtualAddress,
					pWriteDeviceDataInParams->data.vusDataBuf,
					ulItemCount);
				break;

			case 4:
				// Write <ulItemCount> dwords.
				WRITE_REGISTER_BUFFER_ULONG(
					pVirtualAddress,
					pWriteDeviceDataInParams->data.vulDataBuf,
					ulItemCount);
				break;
		}

		// Unmap the virtual address.
		MmUnmapIoSpace(pVirtualAddress, ulNumBytes);
	}
	else
	{
		PUCHAR pucPhysicalAddress = (PUCHAR) ulPhysicalAddress;
		PUSHORT pusPhysicalAddress = (PUSHORT) ulPhysicalAddress;
		PULONG pulPhysicalAddress = (PULONG) ulPhysicalAddress;

		// Write <ulItemCount> <ulItemSize>-sized items to address <ulOffset> in I/O space.
		switch(ulItemSize)
		{
			case 1:
				// Write <ulItemCount> bytes.
				for(ulIndex=0; ulIndex<ulItemCount; ulIndex++)
				{
					WRITE_PORT_UCHAR(pucPhysicalAddress + ulIndex, pWriteDeviceDataInParams->data.vucDataBuf[ulIndex]);
				}
				break;

			case 2:
				// Write <ulItemCount> words.
				for(ulIndex=0; ulIndex<ulItemCount; ulIndex++)
				{
					WRITE_PORT_USHORT(pusPhysicalAddress + ulIndex, pWriteDeviceDataInParams->data.vusDataBuf[ulIndex]);
				}
				break;

			case 4:
				// Write <ulItemCount> dwords.
				for(ulIndex=0; ulIndex<ulItemCount; ulIndex++)
				{
					WRITE_PORT_ULONG(pulPhysicalAddress + ulIndex, pWriteDeviceDataInParams->data.vulDataBuf[ulIndex]);
				}
				break;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  pciexdrvDeviceIoControlDispatch
//      Dispatch routine for IRP_MJ_DEVICE_CONTROL requests.
//
//  Arguments:
//      IN  DeviceObject
//              pointer to the device object for our device
//
//      IN  Irp
//              the device i/o control IRP
//
//  Return Value:
//      NT status code.
//
NTSTATUS pciexdrvDeviceIoControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    PIO_STACK_LOCATION              irpStack;
    NTSTATUS                        status;
    PPCIEXDRV_DEVICE_EXTENSION    deviceExtension;
    PVOID                           inputBuffer;
    ULONG                           inputLength;
    PVOID                           outputBuffer;
    ULONG                           outputLength;

	PREAD_CONFIG_SPACE_DWORD_INPARAMS pReadConfigSpaceDwordInParams;
	PREAD_CONFIG_SPACE_DWORD_OUTPARAMS pReadConfigSpaceDwordOutParams;
	ULONG ulOldConfigAddress, ulConfigAddress;

	PREAD_CONFIG_SPACE_HEADER_INPARAMS pReadConfigSpaceHeaderInParams;
	PREAD_CONFIG_SPACE_HEADER_OUTPARAMS pReadConfigSpaceHeaderOutParams;
	ULONG ulIndex;
	ULONG ulBus, ulDevice, ulFunction;

	PWRITE_CONFIG_SPACE_XXX_INPARAMS pWriteConfigSpaceXxxInParams;

	PREAD_CONFIG_SPACE_XXX_INPARAMS pReadConfigSpaceXxxInParams;
	PREAD_CONFIG_SPACE_XXX_OUTPARAMS pReadConfigSpaceXxxOutParams;

	PREAD_DEVICE_DATA_INPARAMS pReadDeviceDataInParams;
	PREAD_DEVICE_DATA_OUTPARAMS pReadDeviceDataOutParams;
	ULONG ulBytesToRead;

	PWRITE_DEVICE_DATA_INPARAMS pWriteDeviceDataInParams;
	ULONG ulBytesToWrite, ulBytesNeeded;

    pciexdrvDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PPCIEXDRV_DEVICE_EXTENSION)DeviceObject->DeviceExtension;

    // Get our IRP stack location
    irpStack = IoGetCurrentIrpStackLocation(Irp);

    // Get the buffer lengths
    inputLength = irpStack->Parameters.DeviceIoControl.InputBufferLength;
    outputLength = irpStack->Parameters.DeviceIoControl.OutputBufferLength;

    switch (irpStack->Parameters.DeviceIoControl.IoControlCode) 
    {

    case IOCTL_READ_CONFIG_SPACE_DWORD:
        pciexdrvDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_READ_CONFIG_SPACE_DWORD");

		// Validate the input and output buffer pointers and buffer sizes.
		ASSERT(Irp->AssociatedIrp.SystemBuffer);
		ASSERT(inputLength >= sizeof(READ_CONFIG_SPACE_DWORD_INPARAMS));
		ASSERT(outputLength >= sizeof(READ_CONFIG_SPACE_DWORD_OUTPARAMS));

		if( Irp->AssociatedIrp.SystemBuffer == NULL ||
			inputLength < sizeof(READ_CONFIG_SPACE_DWORD_INPARAMS) ||
			outputLength < sizeof(READ_CONFIG_SPACE_DWORD_OUTPARAMS))
		{
			// The input or output buffer is invalid.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

        // Get the in and out parameter structures.
        pReadConfigSpaceDwordInParams = (PREAD_CONFIG_SPACE_DWORD_INPARAMS) Irp->AssociatedIrp.SystemBuffer;
        pReadConfigSpaceDwordOutParams = (PREAD_CONFIG_SPACE_DWORD_OUTPARAMS) Irp->AssociatedIrp.SystemBuffer;
	
		// Read the addressed DWORD from the configuration data port of the host/PCI bridge.
		pReadConfigSpaceDwordOutParams->ulData = ReadConfigDword(
			pReadConfigSpaceDwordInParams->ulBus,
			pReadConfigSpaceDwordInParams->ulDevice,
			pReadConfigSpaceDwordInParams->ulFunction,
			pReadConfigSpaceDwordInParams->ulDwordOffset);

		// Complete the IRP.
		status = STATUS_SUCCESS;
        Irp->IoStatus.Information = sizeof(READ_CONFIG_SPACE_DWORD_OUTPARAMS);
        Irp->IoStatus.Status = status;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);

        break;

    case IOCTL_WRITE_CONFIG_SPACE_XXX:
        pciexdrvDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_WRITE_CONFIG_SPACE_XXX");

		// Validate the input and output buffer pointers and buffer sizes.
		ASSERT(Irp->AssociatedIrp.SystemBuffer);
		ASSERT(inputLength >= sizeof(WRITE_CONFIG_SPACE_XXX_INPARAMS));

		if( Irp->AssociatedIrp.SystemBuffer == NULL ||
			inputLength < sizeof(WRITE_CONFIG_SPACE_XXX_INPARAMS))
		{
			// The input buffer is invalid.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

		// Get the in and out parameter structures.
        pWriteConfigSpaceXxxInParams = (PWRITE_CONFIG_SPACE_XXX_INPARAMS) Irp->AssociatedIrp.SystemBuffer;
	
		// Write the addressed BYTE, WORD or DWORD to the configuration data port of the host/PCI bridge.
		WriteConfigXxx(
			pWriteConfigSpaceXxxInParams->ulBus,
			pWriteConfigSpaceXxxInParams->ulDevice,
			pWriteConfigSpaceXxxInParams->ulFunction,
			pWriteConfigSpaceXxxInParams->ulByteOffset,
			pWriteConfigSpaceXxxInParams->ucSize,
			pWriteConfigSpaceXxxInParams->ulData);

		// Complete the IRP.
		status = STATUS_SUCCESS;
        Irp->IoStatus.Information = 0;
        Irp->IoStatus.Status = status;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);

        break;

	case IOCTL_READ_CONFIG_SPACE_HEADER:
		pciexdrvDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_READ_CONFIG_SPACE_HEADER");

		// Validate the input and output buffer pointers and buffer sizes.
		ASSERT(Irp->AssociatedIrp.SystemBuffer);
		ASSERT(inputLength >= sizeof(READ_CONFIG_SPACE_HEADER_INPARAMS));
		ASSERT(outputLength >= sizeof(READ_CONFIG_SPACE_HEADER_OUTPARAMS));

		if( Irp->AssociatedIrp.SystemBuffer == NULL ||
			inputLength < sizeof(READ_CONFIG_SPACE_HEADER_INPARAMS) ||
			outputLength < sizeof(READ_CONFIG_SPACE_HEADER_OUTPARAMS))
		{
			// The input or output buffer is invalid.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

		// Get the in and out parameter structures.
        pReadConfigSpaceHeaderInParams = (PREAD_CONFIG_SPACE_HEADER_INPARAMS) Irp->AssociatedIrp.SystemBuffer;
        pReadConfigSpaceHeaderOutParams = (PREAD_CONFIG_SPACE_HEADER_OUTPARAMS) Irp->AssociatedIrp.SystemBuffer;
	
		// Save the inparams since we'll overwrite them as the outparams 
		// are copied into the output buffer (shared I/O manager buffer).
		ulBus = pReadConfigSpaceHeaderInParams->ulBus;
		ulDevice = pReadConfigSpaceHeaderInParams->ulDevice;
		ulFunction = pReadConfigSpaceHeaderInParams->ulFunction;

		// Read the 64 ULONGs from the specified function.
		// Note: 16 DWORDS for header and 48 for the rest of the 64 DWORD space.
		for(ulIndex=0; ulIndex<64; ulIndex++)
		{
			pReadConfigSpaceHeaderOutParams->Hdr.vulConfig[ulIndex] = ReadConfigDword(
				ulBus, ulDevice, ulFunction, ulIndex);
		}

		// All ok.
		return CompleteIrp(Irp, sizeof(READ_CONFIG_SPACE_HEADER_OUTPARAMS), STATUS_SUCCESS);
		
		break;

    case IOCTL_READ_CONFIG_SPACE_XXX:
        pciexdrvDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_READ_CONFIG_SPACE_XXX");

		// Validate the input and output buffer pointers and buffer sizes.
		ASSERT(Irp->AssociatedIrp.SystemBuffer);
		ASSERT(inputLength >= sizeof(READ_CONFIG_SPACE_XXX_INPARAMS));
		ASSERT(outputLength >= sizeof(READ_CONFIG_SPACE_XXX_OUTPARAMS));

		if( Irp->AssociatedIrp.SystemBuffer == NULL ||
			inputLength < sizeof(READ_CONFIG_SPACE_XXX_INPARAMS) ||
			outputLength < sizeof(READ_CONFIG_SPACE_XXX_OUTPARAMS))
		{
			// The input or output buffer is invalid.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

        // Get the in and out parameter structures.
        pReadConfigSpaceXxxInParams = (PREAD_CONFIG_SPACE_XXX_INPARAMS) Irp->AssociatedIrp.SystemBuffer;
        pReadConfigSpaceXxxOutParams = (PREAD_CONFIG_SPACE_XXX_OUTPARAMS) Irp->AssociatedIrp.SystemBuffer;
	
		// Read the addressed BYTE, WORD or DWORD from the configuration data port of the host/PCI bridge.
		pReadConfigSpaceXxxOutParams->ulData = ReadConfigXxx(
			pReadConfigSpaceXxxInParams->ulBus,
			pReadConfigSpaceXxxInParams->ulDevice,
			pReadConfigSpaceXxxInParams->ulFunction,
			pReadConfigSpaceXxxInParams->ulByteOffset,
			pReadConfigSpaceXxxInParams->ucSize);

		// Complete the IRP.
		status = STATUS_SUCCESS;
        Irp->IoStatus.Information = sizeof(READ_CONFIG_SPACE_XXX_OUTPARAMS);
        Irp->IoStatus.Status = status;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);

        break;

    case IOCTL_READ_DEVICE_DATA:
        pciexdrvDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_READ_DEVICE_DATA");

		// Validate the input buffer pointer and buffer size.
		ASSERT(Irp->AssociatedIrp.SystemBuffer);
		ASSERT(inputLength >= sizeof(READ_DEVICE_DATA_INPARAMS));

		if( Irp->AssociatedIrp.SystemBuffer == NULL ||
			inputLength < sizeof(READ_DEVICE_DATA_INPARAMS))
		{
			// The input buffer is invalid.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

        // Get the in and out parameter structures.
        pReadDeviceDataInParams = (PREAD_DEVICE_DATA_INPARAMS) Irp->AssociatedIrp.SystemBuffer;
        pReadDeviceDataOutParams = (PREAD_DEVICE_DATA_OUTPARAMS) Irp->AssociatedIrp.SystemBuffer;

		// Find out how many bytes we are reading from the device 
		// and make sure that the output buffer is large enough.
		ulBytesToRead = pReadDeviceDataInParams->ulItemSize * pReadDeviceDataInParams->ulItemCount;
		ASSERT(outputLength >= ulBytesToRead);

		if(outputLength < ulBytesToRead)
		{
			// The output buffer is too small.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

		// Read the data from the device.
		ReadDeviceData(pReadDeviceDataInParams,	pReadDeviceDataOutParams);

		// Complete the IRP.
		status = STATUS_SUCCESS;
        Irp->IoStatus.Information = ulBytesToRead;
        Irp->IoStatus.Status = status;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);
		
		break;

   case IOCTL_WRITE_DEVICE_DATA:
        pciexdrvDebugPrint(DBG_IO, DBG_INFO, __FUNCTION__": IOCTL_WRITE_DEVICE_DATA");

		// Validate the input buffer pointer and buffer size. NOTE: We only check the 
		// fixed size of the variable sized structure. We'll check that the data buffer 
		// is large enough to hold the data to be written further down.
		ASSERT(Irp->AssociatedIrp.SystemBuffer);
		ASSERT(inputLength >= sizeof(WRITE_DEVICE_DATA_INPARAMS));

		if( Irp->AssociatedIrp.SystemBuffer == NULL ||
			inputLength < sizeof(WRITE_DEVICE_DATA_INPARAMS))
		{
			// The input buffer is invalid.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

        // Get the in parameter structure.
        pWriteDeviceDataInParams = (PWRITE_DEVICE_DATA_INPARAMS) Irp->AssociatedIrp.SystemBuffer;

		// Find out how many bytes we are writing to the device and make sure that 
		// the input buffer is large enough to hold all data we are to write.
		ulBytesToWrite = pWriteDeviceDataInParams->ulItemSize * pWriteDeviceDataInParams->ulItemCount;
		ulBytesNeeded = sizeof(WRITE_DEVICE_DATA_INPARAMS) + ulBytesToWrite;
		ASSERT(inputLength >= ulBytesNeeded);

		if(inputLength < ulBytesNeeded)
		{
			// The input buffer is too small.
			return CompleteIrp(Irp, 0, STATUS_INVALID_USER_BUFFER);
		}

		// Write the data to the device.
		WriteDeviceData(pWriteDeviceDataInParams);

		// Complete the IRP.
		status = STATUS_SUCCESS;
        Irp->IoStatus.Information = 0;
        Irp->IoStatus.Status = status;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);
		
		break;

    default:
        status = STATUS_INVALID_DEVICE_REQUEST;
        Irp->IoStatus.Status = status;
        IoCompleteRequest (Irp, IO_NO_INCREMENT);
        break;
    }

    pciexdrvDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  pciexdrvShutdownDispatch
//      Dispatch routine for IRP_MJ_SHUTDOWN requests.
//
//  Arguments:
//      IN  DeviceObject
//              pointer to the device object for our device
//
//      IN  Irp
//              the shutdown IRP
//
//  Return Value:
//      NT status code.
//
NTSTATUS pciexdrvShutdownDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    )
{
    NTSTATUS                        status;
    PPCIEXDRV_DEVICE_EXTENSION    deviceExtension;

    pciexdrvDebugPrint(DBG_GENERAL, DBG_TRACE, __FUNCTION__"++. IRP %p", Irp);

    deviceExtension = (PPCIEXDRV_DEVICE_EXTENSION)DeviceObject->DeviceExtension;

    status = STATUS_NOT_IMPLEMENTED;

    Irp->IoStatus.Status = status;
    Irp->IoStatus.Information = 0;
    IoCompleteRequest (Irp, IO_NO_INCREMENT);

    pciexdrvDebugPrint(DBG_GENERAL, DBG_TRACE, __FUNCTION__"--. IRP %p STATUS %x", Irp, status);

    return status;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//  pciexdrvFlushQueues
//      Flush oustanding IRPs for closed file object.
//
//  Arguments:
//      IN  DeviceExtension
//              our device extension
//
//      IN  FileObject
//              about to be closed file object
//
//  Return Value:
//      none
//
VOID pciexdrvFlushQueues(
    IN  PPCIEXDRV_DEVICE_EXTENSION   DeviceExtension,
    IN  PFILE_OBJECT            FileObject
    )
{
    ULONG   index;

    pciexdrvDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"++");

    pciexdrvDebugPrint(DBG_IO, DBG_TRACE, __FUNCTION__"--");

    return;
}
