/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

#ifndef __PCIEXDRV_H__
#define __PCIEXDRV_H__

// Memory allocation pool tag
#define PCIEXDRV_POOL_TAG 'eicp'

// Make all pool allocations tagged
#undef ExAllocatePool
#define ExAllocatePool(type, size) \
    ExAllocatePoolWithTag(type, size, PCIEXDRV_POOL_TAG);

// queue start io callback
typedef VOID (*PPCIEXDRV_QUEUE_STARTIO)(
    IN PDEVICE_OBJECT   DeviceObject,
    IN PIRP             Irp 
    );

// irp queue type definition
typedef struct _PCIEXDRV_QUEUE
{
    PPCIEXDRV_QUEUE_STARTIO StartIoRoutine;
    PDEVICE_OBJECT  DeviceObject;
    LIST_ENTRY      IrpQueue;
    KSPIN_LOCK      QueueLock;
    PIRP            CurrentIrp;
    KEVENT          StopEvent;
    LONG            StallCount; 
    NTSTATUS        ErrorStatus;
    KDPC            pciexdrvStartIoDpc;
    BOOLEAN         bUsepciexdrvStartIoDpc;
} PCIEXDRV_QUEUE, *PPCIEXDRV_QUEUE;

// cancel-safe irp list type definition
typedef struct _PCIEXDRV_LIST
{
    PDEVICE_OBJECT  DeviceObject;
    LIST_ENTRY      IrpList;
    KSPIN_LOCK      ListLock;
    NTSTATUS        ErrorStatus;
} PCIEXDRV_LIST, *PPCIEXDRV_LIST;

// stall IRP list to syncronize Pnp, Power with
// the rest of IO
typedef struct _PCIEXDRV_IO_LOCK
{
    PDEVICE_OBJECT  DeviceObject;       // our device object
    KEVENT          StallCompleteEvent; // io stalled event
    LIST_ENTRY      StallIrpList;       // stalled irps
    KSPIN_LOCK      IoLock;             // spin lock to syncronize io with stall/unstall
    LONG            StallCount;         // number of times stall was requested
    LONG            ActiveIrpCount;     // number of oustanding, not-stalled IRPs
    NTSTATUS        ErrorStatus;
    PIRP            CurrentIrp;         // used by unstall code
} PCIEXDRV_IO_LOCK, *PPCIEXDRV_IO_LOCK;

// global (per driver) data block
typedef struct _PCIEXDRV_DATA
{
    UNICODE_STRING      RegistryPath;
} PCIEXDRV_DATA, *PPCIEXDRV_DATA;

extern PCIEXDRV_DATA g_Data;

// The device extension for the device object
typedef struct _PCIEXDRV_DEVICE_EXTENSION
{
    PDEVICE_OBJECT          DeviceObject;
    LONG                    OpenHandleCount;
} PCIEXDRV_DEVICE_EXTENSION, *PPCIEXDRV_DEVICE_EXTENSION;


#ifdef __cplusplus
extern "C" {
#endif

// prototype for DriverEntry function
NTSTATUS __stdcall DriverEntry(
    IN  PDRIVER_OBJECT  DriverObject,
    IN  PUNICODE_STRING RegistryPath
    );

VOID __stdcall pciexdrvUnload(
    IN  PDRIVER_OBJECT  DriverObject
    );

NTSTATUS __stdcall pciexdrvDeviceIoControlDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall pciexdrvCreateDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall pciexdrvCloseDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall pciexdrvCleanupDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

NTSTATUS __stdcall pciexdrvShutdownDispatch(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

#ifdef __cplusplus
}
#endif

VOID pciexdrvFlushQueues(
    IN  PPCIEXDRV_DEVICE_EXTENSION   DeviceExtension,
    IN  PFILE_OBJECT            FileObject
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// Registry
///////////////////////////////////////////////////////////////////////////////////////////////////

PVOID pciexdrvRegQueryValueKey(
    IN  HANDLE  RegKeyHandle,
    IN  PWSTR   SubKeyName,
    IN  PWSTR   ValueName,
    OUT PULONG  Length
    );

VOID pciexdrvRegEnumerateKeys(
    IN  HANDLE RegKeyHandle
    );

VOID pciexdrvRegEnumerateValueKeys(
    IN  HANDLE  RegKeyHandle
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// Debug
///////////////////////////////////////////////////////////////////////////////////////////////////

// definition of debug levels

#define DBG_NONE            0
#define DBG_ERR             1
#define DBG_WARN            2
#define DBG_TRACE           3
#define DBG_INFO            4
#define DBG_VERB            5

#ifdef PCIEXDRV_WMI_TRACE

/*
tracepdb -f objchk_wxp_x86\i386\pciexdrv.pdb -p C:\pciexdrv
SET TRACE_FORMAT_SEARCH_PATH=C:\pciexdrv

tracelog -start pciexdrv -guid pciexdrv.ctl -f pciexdrv.log -flags 0x7FFFFFFF -level 5
tracelog -stop pciexdrv

tracefmt -o pciexdrv.txt -f pciexdrv.log
*/

#define WPP_AREA_LEVEL_LOGGER(Area,Lvl)           WPP_LEVEL_LOGGER(Area)
#define WPP_AREA_LEVEL_ENABLED(Area,Lvl)          (WPP_LEVEL_ENABLED(Area) && WPP_CONTROL(WPP_BIT_##Area).Level >= Lvl)

#define WPP_CONTROL_GUIDS \
    WPP_DEFINE_CONTROL_GUID(pciexdrv,(4993ECA0,5E12,4E0E,9482,148CE3CBA510), \
        WPP_DEFINE_BIT(DBG_GENERAL)                 /* bit  0 = 0x00000001 */ \
        WPP_DEFINE_BIT(DBG_PNP)                     /* bit  1 = 0x00000002 */ \
        WPP_DEFINE_BIT(DBG_POWER)                   /* bit  2 = 0x00000004 */ \
        WPP_DEFINE_BIT(DBG_COUNT)                   /* bit  3 = 0x00000008 */ \
        WPP_DEFINE_BIT(DBG_CREATECLOSE)             /* bit  4 = 0x00000010 */ \
        WPP_DEFINE_BIT(DBG_WMI)                     /* bit  5 = 0x00000020 */ \
        WPP_DEFINE_BIT(DBG_UNLOAD)                  /* bit  6 = 0x00000040 */ \
        WPP_DEFINE_BIT(DBG_IO)                      /* bit  7 = 0x00000080 */ \
        WPP_DEFINE_BIT(DBG_INIT)                    /* bit  8 = 0x00000100 */ \
        WPP_DEFINE_BIT(DBG_09)                      /* bit  9 = 0x00000200 */ \
        WPP_DEFINE_BIT(DBG_10)                      /* bit 10 = 0x00000400 */ \
        WPP_DEFINE_BIT(DBG_11)                      /* bit 11 = 0x00000800 */ \
        WPP_DEFINE_BIT(DBG_12)                      /* bit 12 = 0x00001000 */ \
        WPP_DEFINE_BIT(DBG_13)                      /* bit 13 = 0x00002000 */ \
        WPP_DEFINE_BIT(DBG_14)                      /* bit 14 = 0x00004000 */ \
        WPP_DEFINE_BIT(DBG_15)                      /* bit 15 = 0x00008000 */ \
        WPP_DEFINE_BIT(DBG_16)                      /* bit 16 = 0x00010000 */ \
        WPP_DEFINE_BIT(DBG_17)                      /* bit 17 = 0x00020000 */ \
        WPP_DEFINE_BIT(DBG_18)                      /* bit 18 = 0x00040000 */ \
        WPP_DEFINE_BIT(DBG_19)                      /* bit 19 = 0x00080000 */ \
        WPP_DEFINE_BIT(DBG_20)                      /* bit 20 = 0x00100000 */ \
        WPP_DEFINE_BIT(DBG_21)                      /* bit 21 = 0x00200000 */ \
        WPP_DEFINE_BIT(DBG_22)                      /* bit 22 = 0x00400000 */ \
        WPP_DEFINE_BIT(DBG_23)                      /* bit 23 = 0x00800000 */ \
        WPP_DEFINE_BIT(DBG_24)                      /* bit 24 = 0x01000000 */ \
        WPP_DEFINE_BIT(DBG_25)                      /* bit 25 = 0x02000000 */ \
        WPP_DEFINE_BIT(DBG_26)                      /* bit 26 = 0x04000000 */ \
        WPP_DEFINE_BIT(DBG_27)                      /* bit 27 = 0x08000000 */ \
        WPP_DEFINE_BIT(DBG_28)                      /* bit 28 = 0x10000000 */ \
        WPP_DEFINE_BIT(DBG_29)                      /* bit 29 = 0x20000000 */ \
        WPP_DEFINE_BIT(DBG_30)                      /* bit 30 = 0x40000000 */ \
        WPP_DEFINE_BIT(DBG_31)                      /* bit 31 = 0x80000000 */ \
        )

__inline VOID pciexdrvDumpIrp(
    IN PIRP Irp
    )
{
}

__inline PCHAR SystemPowerStateString(
    IN  SYSTEM_POWER_STATE  SystemState
    )
{
    return "";
}

__inline PCHAR DevicePowerStateString(
    IN  DEVICE_POWER_STATE  DeviceState
    )
{
    return "";
}

#else

// definition of debug areas

#define DBG_GENERAL         (1 << 0)
#define DBG_PNP             (1 << 1)
#define DBG_POWER           (1 << 2)
#define DBG_COUNT           (1 << 3)
#define DBG_CREATECLOSE     (1 << 4)
#define DBG_WMI             (1 << 5)
#define DBG_UNLOAD          (1 << 6)
#define DBG_IO              (1 << 7)
#define DBG_INIT            (1 << 8)

#define DBG_ALL             0xFFFFFFFF

#if DBG

VOID pciexdrvDebugPrint(
    IN ULONG    Area,
    IN ULONG    Level,
    IN PCCHAR   Format,
    IN          ...
    );

VOID pciexdrvDumpIrp(
    IN PIRP Irp
    );

PCHAR IrpMajorFunctionString(
    IN  UCHAR MajorFunction
    );

PCHAR PnPMinorFunctionString(
    IN  UCHAR   MinorFunction
    );

PCHAR PowerMinorFunctionString(
    IN  UCHAR   MinorFunction
    );

PCHAR SystemPowerStateString(
    IN  SYSTEM_POWER_STATE  SystemState
    );

PCHAR DevicePowerStateString(
    IN  DEVICE_POWER_STATE  DeviceState
    );

PCHAR WMIMinorFunctionString (
    IN  UCHAR MinorFunction
    );

#else	// !DBG

__inline VOID pciexdrvDebugPrint(
    IN ULONG    Area,
    IN ULONG    Level,
    IN PCCHAR   Format,
    IN          ...
    )
{
}

#define pciexdrvDumpIrp(Irp)
#define IrpMajorFunctionString(MajorFunction)
#define PnPMinorFunctionString(MinorFunction)
#define PowerMinorFunctionString(MinorFunction)
#define SystemPowerStateString(SystemState) 0
#define DevicePowerStateString(DeviceState) 0
#define WMIMinorFunctionString(MinorFunction)
#define pciexdrvTdiDumpAddress(Address)

#endif	// DBG
#endif	// PCIEXDRV_WMI_TRACE

///////////////////////////////////////////////////////////////////////////////////////////////////
// Queue Functions
///////////////////////////////////////////////////////////////////////////////////////////////////

VOID pciexdrvInitializeQueue(
    IN  PPCIEXDRV_QUEUE    QueueExtension,
    IN  PPCIEXDRV_QUEUE_STARTIO     StartIoRoutine,
    IN  PDEVICE_OBJECT      DeviceObject,
    IN  BOOLEAN             bUsepciexdrvStartIoDpc
    );

NTSTATUS pciexdrvQueueIrp(
    IN  PPCIEXDRV_QUEUE    QueueExtension,
    IN  PIRP                Irp
    );

VOID pciexdrvStartNext(
    IN  PPCIEXDRV_QUEUE    QueueExtension
    );

VOID pciexdrvFlushQueue(
    IN  PPCIEXDRV_QUEUE    QueueExtension,
    IN  PFILE_OBJECT        FileObject
    );

VOID pciexdrvInvalidateQueue(
    IN  PPCIEXDRV_QUEUE    Queue,
    IN  NTSTATUS                  ErrorStatus
    );

VOID pciexdrvPauseQueue(
    IN  PPCIEXDRV_QUEUE    QueueExtension
    );
    
VOID pciexdrvRestartQueue(
    IN  PPCIEXDRV_QUEUE    QueueExtension
    );

VOID pciexdrvStartIoDpc(
    IN  PKDPC       Dpc,
    IN  PVOID       Context,
    IN  PVOID       Unused1,
    IN  PVOID       Unused2
    );

VOID pciexdrvQueueCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

///////////////////////////////////////////////////////////////////////////////////////////////////
// List functions
///////////////////////////////////////////////////////////////////////////////////////////////////

VOID pciexdrvInitializeList(
    IN  PPCIEXDRV_LIST   List,
    IN  PDEVICE_OBJECT          DeviceObject
    );

NTSTATUS pciexdrvInsertHead(
    IN  PPCIEXDRV_LIST   List, 
    IN  PIRP                Irp
    );

NTSTATUS pciexdrvInsertTail(
    IN  PPCIEXDRV_LIST   List, 
    IN  PIRP                Irp
    );

PIRP pciexdrvRemoveHead(
    IN  PPCIEXDRV_LIST   List
    );

PIRP pciexdrvRemoveTail(
    IN  PPCIEXDRV_LIST   List
    );

VOID pciexdrvFlushList(
    IN  PPCIEXDRV_LIST   List,
    IN  PFILE_OBJECT        FileObject
    );

VOID pciexdrvInvalidateList(
    IN  PPCIEXDRV_LIST   List,
    IN  NTSTATUS                ErrorStatus
    );

VOID pciexdrvListCancelRoutine(
    IN  PDEVICE_OBJECT  DeviceObject,
    IN  PIRP            Irp
    );

#endif  // __PCIEXDRV_H__
