/**********************************************************************
;       Copyright 2005 Summit Soft Consulting
;
;                       RESTRICTED RIGHTS LEGEND
;                       ------------------------
;
; Use, duplication, or disclosure by any commercial industry (public
; or private), private individual, or by any Government Agency without
; a express written consent of release from Summit Soft Consulting
; is subject to restrictions as set forth in paragraph (b) (3) (B) of
; the Rights in Technical Data and Computer Software clause in
; DAR 7-104.9 (a).  Manufacturer is: Summit Soft Consulting,
; 26895 Aliso Creek Rd. Suite B504, Aliso Viejo, CA 92656,
; http://www.SummitSoftConsulting.com
;**********************************************************************/

// GUID definition are required to be outside of header inclusion pragma to avoid
// error during precompiled headers.
//

#ifndef __INTRFACE_H__
#define __INTRFACE_H__

////////////////////////////////////////////////////////////
//	Structures common between the driver and application.
///////////////////////////////////////////////////////////

#pragma pack( push , 1 )

typedef struct _TYPE0_HEADER
{
	USHORT usVendorID;
	USHORT usDeviceID;
	USHORT usCommandRegister;
	USHORT usStatusRegister;
	UCHAR ucRevisionID;
	UCHAR ucProgrammingInterface;
	UCHAR ucSubClassCode;
	UCHAR ucClassCode;
	UCHAR ucCacheLineSize;
	UCHAR ucLatencyTimer;
	UCHAR ucHeaderType;
	UCHAR ucBIST;
	ULONG ulBAR0;
	ULONG ulBAR1;
	ULONG ulBAR2;
	ULONG ulBAR3;
	ULONG ulBAR4;
	ULONG ulBAR5;
	ULONG ulCardBusCisPointer;
	USHORT usSubSystemVendorID;
	USHORT usSubSystemDeviceID;
	ULONG ulExpansionRomBAR;
	UCHAR ucCapabilitiesPointer;
	UCHAR vucReserved1[3];
	ULONG ulReserved2;
	UCHAR ucInterruptLine;
	UCHAR ucInterruptPin;
	UCHAR ucMinGnt;
	UCHAR ucMaxLat;

	ULONG vulNewCaps[48];

}TYPE0_HEADER, *PTYPE0_HEADER;

typedef struct _TYPE1_HEADER
{
	USHORT usVendorID;
	USHORT usDeviceID;
	USHORT usCommandRegister;
	USHORT usStatusRegister;
	UCHAR ucRevisionID;
	UCHAR ucProgrammingInterface;
	UCHAR ucSubClassCode;
	UCHAR ucClassCode;
	UCHAR ucCacheLineSize;
	UCHAR ucLatencyTimer;
	UCHAR ucHeaderType;
	UCHAR ucBIST;
	ULONG ulBAR0;
	ULONG ulBAR1;
	UCHAR ucPrimaryBusNumber;
	UCHAR ucSecondaryBusNumber;
	UCHAR ucSubordinateBusNumber;
	UCHAR ucSecondaryLatencyTimer;
	UCHAR ucIoBase;
	UCHAR ucIoLimit;
	USHORT usSecondaryStatus;
	USHORT usMemoryBase;
	USHORT usMemoryLimit;
	USHORT usPrefetchableMemoryBase;
	USHORT usPrefetchableMemoryLimit;
	ULONG ulPrefetchableMemoryBaseUpper32;
	ULONG ulPrefetchableMemoryLimitUpper32;
	USHORT usIoBaseUpper16;
	USHORT usIoLimitUpper16;
	UCHAR ucCapabilitiesPointer;
	UCHAR vucReserved1[3];
	ULONG ulExpansionRomBAR;
	UCHAR ucInterruptLine;
	UCHAR ucInterruptPin;
	USHORT usBridgeControl;

	ULONG vulNewCaps[48];

}TYPE1_HEADER, *PTYPE1_HEADER;

typedef struct _TYPE2_HEADER
{
	USHORT usVendorID;
	USHORT usDeviceID;
	USHORT usCommandRegister;
	USHORT usStatusRegister;
	UCHAR ucRevisionID;
	UCHAR ucProgrammingInterface;
	UCHAR ucSubClassCode;
	UCHAR ucClassCode;
	UCHAR ucCacheLineSize;
	UCHAR ucLatencyTimer;
	UCHAR ucHeaderType;
	UCHAR ucBIST;
	ULONG ulCardbusSocketRegisters;
	UCHAR vucReserved[2];
	USHORT usSecondaryStatus;
	UCHAR ucPciBusNumber;
	UCHAR ucCardBusBusNumber;
	UCHAR ucSubordinateBusNumber;
	UCHAR ucCardbusLatencyTimer;
	ULONG ulMemoryBaseRegister0;
	ULONG ulMemoryLimitRegister0;
	ULONG ulMemoryBaseRegister1;
	ULONG ulMemoryLimitRegister1;
	USHORT usIoBaseRegisterLower0;
	USHORT usIoBaseRegisterUpper0;
	USHORT usIoLimitRegisterLower0;
	USHORT usIoLimitRegisterUpper0;
	USHORT usIoBaseRegisterLower1;
	USHORT usIoBaseRegisterUpper1;
	USHORT usIoLimitRegisterLower1;
	USHORT usIoLimitRegisterUpper1;
	UCHAR ucInterruptLine;
	UCHAR ucInterruptPin;
	USHORT usBridgeControl;
	USHORT usSubSystemVendorID;
	USHORT usSubSystemDeviceID;
	ULONG ulLegacyModeBaseAddress;
	ULONG vulReserved[14];
	ULONG vulVendorSpecific[32];

}TYPE2_HEADER, *PTYPE2_HEADER;

typedef	union _TYPEX_HEADER
{
	TYPE0_HEADER Type0Hdr;	// 'Regular' PCI device.
	TYPE1_HEADER Type1Hdr;	// Pci/Pci Bridge.
	TYPE2_HEADER Type2Hdr;	// CardBus.
	ULONG vulConfig[64];	// Unformatted.

}TYPEX_HEADER, *PTYPEX_HEADER;

#define FILE_DEVICE_PCIEXDRV  0x8000

// Define Interface reference/dereference routines for
// Interfaces exported by IRP_MN_QUERY_INTERFACE

#define PCIEXDRV_IOCTL(index) \
    CTL_CODE(FILE_DEVICE_PCIEXDRV, index, METHOD_BUFFERED, FILE_READ_DATA)

/////////////////////////////////////////////
/// IOCTL_READ_CONFIG_SPACE_DWORD
/////////////////////////////////////////////

#define IOCTL_READ_CONFIG_SPACE_DWORD \
    CTL_CODE(FILE_DEVICE_PCIEXDRV, 0x800, METHOD_BUFFERED, FILE_READ_ACCESS)

// In-parameters.
typedef struct _READ_CONFIG_SPACE_DWORD_INPARAMS
{
	ULONG ulBus;
	ULONG ulDevice;
	ULONG ulFunction;
	ULONG ulDwordOffset;	

}READ_CONFIG_SPACE_DWORD_INPARAMS, *PREAD_CONFIG_SPACE_DWORD_INPARAMS;

// Out-parameters.
typedef struct _READ_CONFIG_SPACE_DWORD_OUTPARAMS
{
	ULONG ulData;

}READ_CONFIG_SPACE_DWORD_OUTPARAMS, *PREAD_CONFIG_SPACE_DWORD_OUTPARAMS;

/////////////////////////////////////////////
/// IOCTL_WRITE_CONFIG_SPACE_XXX
/////////////////////////////////////////////

#define IOCTL_WRITE_CONFIG_SPACE_XXX \
    CTL_CODE(FILE_DEVICE_PCIEXDRV, 0x801, METHOD_BUFFERED, FILE_WRITE_ACCESS)

// In-parameters.
typedef struct _WRITE_CONFIG_SPACE_XXX_INPARAMS
{
	ULONG ulBus;
	ULONG ulDevice;
	ULONG ulFunction;
	ULONG ulByteOffset;
	UCHAR ucSize;
	ULONG ulData;

}WRITE_CONFIG_SPACE_XXX_INPARAMS, *PWRITE_CONFIG_SPACE_XXX_INPARAMS;

// No out-parameters.

/////////////////////////////////////////////
/// IOCTL_READ_CONFIG_SPACE_HEADER
/////////////////////////////////////////////

#define IOCTL_READ_CONFIG_SPACE_HEADER \
    CTL_CODE(FILE_DEVICE_PCIEXDRV, 0x802, METHOD_BUFFERED, FILE_READ_ACCESS)

// In-parameters.
typedef struct _READ_CONFIG_SPACE_HEADER_INPARAMS
{
	ULONG ulBus;
	ULONG ulDevice;
	ULONG ulFunction;

}READ_CONFIG_SPACE_HEADER_INPARAMS, *PREAD_CONFIG_SPACE_HEADER_INPARAMS;

// Out-parameters.
typedef struct _READ_CONFIG_SPACE_HEADER_OUTPARAMS
{
	TYPEX_HEADER Hdr;

}READ_CONFIG_SPACE_HEADER_OUTPARAMS, *PREAD_CONFIG_SPACE_HEADER_OUTPARAMS;

/////////////////////////////////////////////
/// IOCTL_READ_CONFIG_SPACE_XXX
/////////////////////////////////////////////

#define IOCTL_READ_CONFIG_SPACE_XXX \
    CTL_CODE(FILE_DEVICE_PCIEXDRV, 0x803, METHOD_BUFFERED, FILE_READ_ACCESS)

// In-parameters.
typedef struct _READ_CONFIG_SPACE_XXX_INPARAMS
{
	ULONG ulBus;
	ULONG ulDevice;
	ULONG ulFunction;
	ULONG ulByteOffset;	
	UCHAR ucSize;

}READ_CONFIG_SPACE_XXX_INPARAMS, *PREAD_CONFIG_SPACE_XXX_INPARAMS;

// Out-parameters.
typedef struct _READ_CONFIG_SPACE_XXX_OUTPARAMS
{
	ULONG ulData;

}READ_CONFIG_SPACE_XXX_OUTPARAMS, *PREAD_CONFIG_SPACE_XXX_OUTPARAMS;

/////////////////////////////////////////////
/// IOCTL_READ_DEVICE_DATA
/////////////////////////////////////////////

#define IOCTL_READ_DEVICE_DATA \
    CTL_CODE(FILE_DEVICE_PCIEXDRV, 0x804, METHOD_BUFFERED, FILE_READ_ACCESS)

// In-parameters.
typedef struct _READ_DEVICE_DATA_INPARAMS
{
	UCHAR ucMemorySpace;		// 0: I/O Space, 1: Mem Space.
	ULONG ulPhysicalAddress;	// Dword-aligned offset to read from.
	ULONG ulItemSize;			// 1: Byte, 2: Word, 4: Dword.
	ULONG ulItemCount;			// Number of bytes, words or dwords.

}READ_DEVICE_DATA_INPARAMS, *PREAD_DEVICE_DATA_INPARAMS;

// Out-parameters.
typedef struct _READ_DEVICE_DATA_OUTPARAMS
{
	union{
		UCHAR vucDataBuf[1];
		USHORT vusDataBuf[1];
		ULONG vulDataBuf[1];
	}data;

}READ_DEVICE_DATA_OUTPARAMS, *PREAD_DEVICE_DATA_OUTPARAMS;

/////////////////////////////////////////////
/// IOCTL_WRITE_DEVICE_DATA
/////////////////////////////////////////////

#define IOCTL_WRITE_DEVICE_DATA \
    CTL_CODE(FILE_DEVICE_PCIEXDRV, 0x805, METHOD_BUFFERED, FILE_WRITE_ACCESS)

// In-parameters.
typedef struct _WRITE_DEVICE_DATA_INPARAMS
{
	UCHAR ucMemorySpace;		// 0: I/O Space, 1: Mem Space.
	ULONG ulPhysicalAddress;	// Dword-aligned offset to write to.
	ULONG ulItemSize;			// 1: Byte, 2: Word, 4: Dword.
	ULONG ulItemCount;			// Number of bytes, words or dwords.

	union{
		UCHAR vucDataBuf[1];
		USHORT vusDataBuf[1];
		ULONG vulDataBuf[1];
	}data;

}WRITE_DEVICE_DATA_INPARAMS, *PWRITE_DEVICE_DATA_INPARAMS;

// Out-parameters.
// None.

#pragma pack( pop )

#endif // __INTRFACE_H__
